// jshint maxerr:1000
/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ===============================================================================================================
   ===============================================================================================================
                                                     ADVANCED TAB TOOL 
   ===============================================================================================================
   =============================================================================================================== */

let Adv_Error = {
    /**
     * Object holding all the error/condition checking functions that apply to both SDK3 and SDK5.
     * If the condition is triggered the function returns a list [1, 'error text']; 
     * otherwise the function returns [0, ''] 
     */
    General_Rules: {
        // Verify a device has been selected
        check_device: function(){
            return (templateObj.$.ti_widget_dropdown_cc_device.selectedValue === undefined)?[1, 'Select a device'] : [0, ''];
        },
        // Verify an evm has been selected
        check_evmBoard: function(){
            return (templateObj.$.ti_widget_dropdown_cc_evm.selectedValue === undefined)?[1, 'Select an EVM'] : [0, ''];
        },
        // Verify an adc mode has been selected
        check_adcMode: function(){
            return (templateObj.$.ti_widget_dropdown_cc_adcMode.selectedValue === undefined)?[1, 'Select an ADC mode'] : [0, ''];
        },
    },
    /**
     * An object holding all the error/condition checking functions for SDK3. 
     * If the condition is triggered the function returns a list [1, 'error text'];
     * otherwise the function returns [0, '']
     * 
     * Applies to devices [xWR1443, xWR1642, xWR1843, xWR6843]
     */
    SDK3_Rules: {
        // Verify that the rxChannel is not too high for the selected device
        check_channelCfg_rxChannelEn: function(){
            // 4 antenna devices SDK 3 devices
            var sdk3_4AntennaRxDevices = ['xWR1443','xWR1642','xWR1843','xWR6843'];
            if (sdk3_4AntennaRxDevices.includes(mmWaveAdvanced.Input.device)) {
                if (mmWaveAdvanced.Input.channelCfg_rxChannelEn > 15){
                    return [1,'rxChannelEn bitmask is trying to use more rx channels than the device has, the max is 0b1111 = 15'];
                }
            }
            else if (mmWaveAdvanced.Input.channelCfg_rxChannelEn <= 0){
                return [1,'rxChannelEn bitmask is trying to use no rx channels, the min is 0b0001 = 1'];
            }
            return [0, ''];
        },
        // Verify that the txChannel is not too high for the selected device
        check_channelCfg_txChannelEn: function(){
            var sdk3_3AntennaTxDevices = ['xWR1443','xWR1843','xWR6843'];
            var sdk3_2AntennaTxDevices = ['xWR1642'];
            if (sdk3_3AntennaTxDevices.includes(mmWaveAdvanced.Input.device)) {
                if (mmWaveAdvanced.Input.channelCfg_txChannelEn > 7){
                    return [1,'txChannelEn bitmask is trying to use more tx channels than the device has, the max is 0b111 = 7'];
                }
            }
            else if (sdk3_2AntennaTxDevices.includes(mmWaveAdvanced.Input.device)) {
                if (mmWaveAdvanced.Input.channelCfg_txChannelEn > 3){
                    return [1,'txChannelEn bitmask is trying to use more tx channels than the device has, the max is 0b11 = 3'];
                }
            }
            else if (mmWaveAdvanced.Input.channelCfg_txChannelEn <= 0){
                return [1,'txChannelEn bitmask is trying to use no tx channels, the min is 0b001 = 1'];
            }
            return [0, ''];
        },
        // Conditionally display chirpCfg1 depending on the bitmask; does not return an error string
        check_chirpCfg1: function(){
            // Check if the first bit in the txChannelEn mask is set
            var mask_chirpCfg1 = 1 << 0;
            var chirpCfg1Inputs = ['ti_widget_textbox_cc_chirpCfg_1_chirpStartIdx',
                                   'ti_widget_textbox_cc_chirpCfg_1_chirpEndIdx',
                                   'ti_widget_textbox_cc_chirpCfg_1_profileId',
                                   'ti_widget_textbox_cc_chirpCfg_1_startFreqVar',
                                   'ti_widget_textbox_cc_chirpCfg_1_freqSlopeVar',
                                   'ti_widget_textbox_cc_chirpCfg_1_idleTimeVar',
                                   'ti_widget_textbox_cc_chirpCfg_1_adcStartTimeVar',
                                   'ti_widget_textbox_cc_chirpCfg_1_txEnableMask'];
            if ((mask_chirpCfg1 & mmWaveAdvanced.Input.channelCfg_txChannelEn) != 0){
                templateObj.$.ti_widget_container_chirpCfg_1.style.visibility = 'visible';
                // When visible, but nothing input, give an error because their configuration is wrong
                var inputCount = 0;
                // Is every cell in chirpCfg blank?
                chirpCfg1Inputs.forEach(textbox => {
                    if (templateObj.$[textbox].value !== '') inputCount += 1;
                });
                if (inputCount == 0) {
                    return [1,'txChannelEn bitmask is trying to use chirpCfg 1, but you have no values here. Change txChannelEn or fill out chirpCfg 1.'];
                }
            } else {
                templateObj.$.ti_widget_container_chirpCfg_1.style.visibility = 'hidden';
            }
            return [0, ''];
        },
        // Conditionally display chirpCfg2 depending on the bitmask; does not return an error string
        check_chirpCfg2: function(){
            // Check if the second bit in the txChannelEn mask is set
            var mask_chirpCfg2 = 1 << 1;
            var chirpCfg2Inputs = ['ti_widget_textbox_cc_chirpCfg_2_chirpStartIdx',
                                   'ti_widget_textbox_cc_chirpCfg_2_chirpEndIdx',
                                   'ti_widget_textbox_cc_chirpCfg_2_profileId',
                                   'ti_widget_textbox_cc_chirpCfg_2_startFreqVar',
                                   'ti_widget_textbox_cc_chirpCfg_2_freqSlopeVar',
                                   'ti_widget_textbox_cc_chirpCfg_2_idleTimeVar',
                                   'ti_widget_textbox_cc_chirpCfg_2_adcStartTimeVar',
                                   'ti_widget_textbox_cc_chirpCfg_2_txEnableMask'];
            if ((mask_chirpCfg2 & mmWaveAdvanced.Input.channelCfg_txChannelEn) != 0){
                templateObj.$.ti_widget_container_chirpCfg_2.style.visibility = 'visible';
                // When visible, but nothing input, give an error because their configuration is wrong
                var inputCount = 0;
                // Is every cell in chirpCfg blank?
                chirpCfg2Inputs.forEach(textbox => {
                    if (templateObj.$[textbox].value !== '') inputCount += 1;
                });
                if (inputCount == 0) {
                    return [1,'txChannelEn bitmask is trying to use chirpCfg 2, but you have no values here. Change txChannelEn or fill out chirpCfg 2.'];
                }
            } else {
                templateObj.$.ti_widget_container_chirpCfg_2.style.visibility = 'hidden';
            }
            return [0, ''];
        },
        // Conditionally display chirpCfg3 depending on the bitmask; does not return an error string
        check_chirpCfg3: function(){
            // Check if the third bit in the txChannelEn mask is set
            var mask_chirpCfg3 = 1 << 2;
            var chirpCfg3Inputs = ['ti_widget_textbox_cc_chirpCfg_3_chirpStartIdx',
                                   'ti_widget_textbox_cc_chirpCfg_3_chirpEndIdx',
                                   'ti_widget_textbox_cc_chirpCfg_3_profileId',
                                   'ti_widget_textbox_cc_chirpCfg_3_startFreqVar',
                                   'ti_widget_textbox_cc_chirpCfg_3_freqSlopeVar',
                                   'ti_widget_textbox_cc_chirpCfg_3_idleTimeVar',
                                   'ti_widget_textbox_cc_chirpCfg_3_adcStartTimeVar',
                                   'ti_widget_textbox_cc_chirpCfg_3_txEnableMask'];
            if ((mask_chirpCfg3 & mmWaveAdvanced.Input.channelCfg_txChannelEn) != 0){
                templateObj.$.ti_widget_container_chirpCfg_3.style.visibility = 'visible';
                // When visible, but nothing input, give an error because their configuration is wrong
                var inputCount = 0;
                // Is every cell in chirpCfg blank?
                chirpCfg3Inputs.forEach(textbox => {
                    if (templateObj.$[textbox].value !== '') inputCount += 1;
                });
                if (inputCount == 0) {
                    return [1,'txChannelEn bitmask is trying to use chirpCfg 3, but you have no values here. Change txChannelEn or fill out chirpCfg 3.'];
                }
            } else {
                templateObj.$.ti_widget_container_chirpCfg_3.style.visibility = 'hidden';
            }
            return [0, ''];
        },
        // Warn on the numSamples lower bound
        check_numSamples: function(){
            return (mmWaveAdvanced.Output.chirpParam_numSamples < 64)?[1, 'Suggested to have number of samples >= 64'] : [0, ''];
        },
        // Warn on the frequencySlope upper bound
        check_frequencySlope: function(){
            if (mmWaveAdvanced.Output.chirpParam_frequencySlope >= 250) {
                [1, 'Suggested to have slope < 250'];
            }
            else if (mmWaveAdvanced.Output.chirpParam_frequencySlope < 0) {
                [1, 'Suggested to have slope > 0'];
            }
            return [0, ''];
        },
        // Verify the validSweepBandwidth bounds
        check_validSweepBandwidth: function(){
            // Handle the unique hardware case of IWR6843 where the 2 VCO impose a piece-wise validSweepBandwidth curve
            if (mmWaveAdvanced.Input.device === 'xWR6843'){
                // When start frequency is lower than 60GHz (57-60), the end frequency must be below 60.75GHz (i.e. a validSweepBandwidth of 3750MHz)
                if (mmWaveAdvanced.Input.profileCfg_startFreq < 60) {
                    if (mmWaveAdvanced.Output.infoOnly_validSweepBandwidth >= 3750) {
                        ['ti_widget_container_cc_numSamples',
                        'ti_widget_container_cc_digOutSampleRate',
                        'ti_widget_container_cc_freqSlope'].forEach(widget => {
                           Util.Html.highlightPropWithPrecedence(templateObj.$[widget],1, 'conditional');
                        });
                        return [1, 'validSweepBandwidth too high (>= 3750); for the xWR6843, if the startFreq is lower than 60GHz, then the end frequency must be below 60.75GHz (i.e. a validSweepBandwidth of 3750MHz): reduce bandwidth by lowering freqSlope, lowering numAdcSamples, and/or increasing digOutSampleRate'];
                    }
                }
                // Otherwise we still have a valid sweep bandwidth of 4000 MHz
                else if (mmWaveAdvanced.Input.profileCfg_startFreq >= 60) {
                    if (mmWaveAdvanced.Output.infoOnly_validSweepBandwidth >= 4000) {
                        ['ti_widget_container_cc_numSamples',
                        'ti_widget_container_cc_digOutSampleRate',
                        'ti_widget_container_cc_freqSlope'].forEach(widget => {
                            Util.Html.highlightPropWithPrecedence(templateObj.$[widget],1, 'conditional');
                        });
                        return [1, 'validSweepBandwidth too high (>= 4000): reduce bandwidth by lowering freqSlope, lowering numAdcSamples, and/or increasing digOutSampleRate'];
                    }
                }
                else if (mmWaveAdvanced.Output.infoOnly_validSweepBandwidth <= 0){
                    ['ti_widget_container_cc_numSamples',
                     'ti_widget_container_cc_digOutSampleRate',
                     'ti_widget_container_cc_freqSlope'].forEach(widget => {
                        Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                    });
                    return [1, 'validSweepBandwidth too low (<= 0): increase bandwidth by raising freqSlope, raising numAdcSamples, and/or decreasing digOutSampleRate'];
                }
                return [0, ''];
            }
            // For all other SDK3 devices, the valid bandwidth is computed normally
            else {
                if (mmWaveAdvanced.Output.infoOnly_validSweepBandwidth >= 4000) {
                    ['ti_widget_container_cc_numSamples',
                     'ti_widget_container_cc_digOutSampleRate',
                     'ti_widget_container_cc_freqSlope'].forEach(widget => {
                        Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                    });
                    return [1, 'validSweepBandwidth too high: reduce bandwidth by lowering freqSlope, lowering numAdcSamples, and/or increasing digOutSampleRate'];
                }
                else if (mmWaveAdvanced.Output.infoOnly_validSweepBandwidth <= 0){
                    ['ti_widget_container_cc_numSamples',
                     'ti_widget_container_cc_digOutSampleRate',
                     'ti_widget_container_cc_freqSlope'].forEach(widget => {
                        Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                    });
                    return [1, 'validSweepBandwidth too low (<= 0): increase bandwidth by raising freqSlope, raising numAdcSamples, and/or decreasing digOutSampleRate'];
                }
                return [0, ''];
            }
        },
        // Verify the rampEndTime condition
        check_rampEndTime: function(){
            if (mmWaveAdvanced.Input.profileCfg_rampEndTime < (mmWaveAdvanced.Input.profileCfg_adcStartTime + mmWaveAdvanced.Output.infoOnly_chirpTime)) {
                ['ti_widget_container_cc_adcStartTime',
                 'ti_widget_container_adcStartTime',
                 'ti_widget_container_chirpTime'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                });
                return [1, 'rampEndTime must be greater than the sum of the adcStartTime and chirpTime'];
            }
            else if (mmWaveAdvanced.Output.results_radarCubeSize <= 0){
                ['ti_widget_container_cc_adcStartTime',
                'ti_widget_container_adcStartTime',
                'ti_widget_container_chirpTime'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                });
                return [1, 'radarCubeSize must be > 0'];
            }
            return [0, ''];
        },
        // Verify the radar cube size will fit into the device's L3 memory
        check_radarCubeSize: function(){
            if (mmWaveAdvanced.Input.device === 'xWR6843'){
                if (mmWaveAdvanced.Output.results_radarCubeSize > 768) {
                    ['ti_widget_container_cc_numLoops',
                     'ti_widget_container_numLoops',
                     'ti_widget_container_cc_numSamples'].forEach(widget => {
                        Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                    });
                    return [1, 'radarCubeSize must be < xWR6843 L3 size (768k): Reduce by lowering numLoops and/or numADCSamples'];
                } 
            }
            else if (mmWaveAdvanced.Input.device === 'xWR1843'){
                if (mmWaveAdvanced.Output.results_radarCubeSize > 1024) {
                    ['ti_widget_container_cc_numLoops',
                     'ti_widget_container_numLoops',
                     'ti_widget_container_cc_numSamples'].forEach(widget => {
                        Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                    });
                    return [1, 'radarCubeSize must be < xWR1843 L3 size (1024k): Reduce by lowering numLoops and/or numADCSamples'];
                }
            }
            else if (mmWaveAdvanced.Input.device === 'xWR1642'){
                if (mmWaveAdvanced.Output.results_radarCubeSize > 768) {
                    ['ti_widget_container_cc_numLoops',
                     'ti_widget_container_numLoops',
                     'ti_widget_container_cc_numSamples'].forEach(widget => {
                        Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                    });
                    return [1, 'radarCubeSize must be < xWR1642 L3 size (768k): Reduce by lowering numLoops and/or numADCSamples'];
                } 
            }
            else if (mmWaveAdvanced.Input.device === 'xWR1443'){
                if (mmWaveAdvanced.Output.results_radarCubeSize > 384) {
                    ['ti_widget_container_cc_numLoops',
                     'ti_widget_container_numLoops',
                     'ti_widget_container_cc_numSamples'].forEach(widget => {
                        Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                    });
                    return [1, 'radarCubeSize must be < xWR1443 L3 size (384k): Reduce by lowering numLoops and/or numADCSamples'];
                } 
            }
            else if (mmWaveAdvanced.Output.results_radarCubeSize <= 0){
                ['ti_widget_container_cc_numLoops',
                 'ti_widget_container_numLoops',
                 'ti_widget_container_cc_numSamples'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                });
                return [1, 'radarCubeSize must be > 0: Increase by raising numLoops and/or numADCSamples'];
            }
            return [0, ''];
        },
        // Verify the sampling rate against the ADC mode and the device selected
        check_samplingRate_upper: function(){
            if (mmWaveAdvanced.Input.device === 'xWR6843'){
                if (mmWaveAdvanced.Output.adcbufCfg_adcOutputFmt === 1){ // Real
                    if (mmWaveAdvanced.Output.chirpParam_samplingRate > 25) return [1, 'xWR6843 real mode sampling rate must be < 25 Msps'];
                } else {
                    if (mmWaveAdvanced.Output.chirpParam_samplingRate > 12.5) return [1, 'xWR6843 complex mode sampling rate must be < 12.5 Msps'];
                }
            }
            else if (mmWaveAdvanced.Input.device === 'xWR1843'){
                if (mmWaveAdvanced.Output.adcbufCfg_adcOutputFmt === 1){ // Real
                    if (mmWaveAdvanced.Output.chirpParam_samplingRate > 25) return [1, 'xWR1843 real mode sampling rate must be < 25 Msps'];
                } else {
                    if (mmWaveAdvanced.Output.chirpParam_samplingRate > 12.5) return [1, 'xWR1843 complex mode sampling rate must be < 12.5 Msps'];
                }
            }
            else if (mmWaveAdvanced.Input.device === 'xWR1642'){
                if (mmWaveAdvanced.Output.adcbufCfg_adcOutputFmt === 1){ // Real
                    if (mmWaveAdvanced.Output.chirpParam_samplingRate > 12.5) return [1, 'xWR1642 real mode sampling rate must be < 12.5 Msps'];
                } else {
                    if (mmWaveAdvanced.Output.chirpParam_samplingRate > 6.25) return [1, 'xWR1642 complex mode sampling rate must be < 6.25 Msps'];
                }
            }
            else if (mmWaveAdvanced.Input.device === 'xWR1443'){
                if (mmWaveAdvanced.Output.adcbufCfg_adcOutputFmt === 1){ // Real
                    if (mmWaveAdvanced.Output.chirpParam_samplingRate > 12.5) return [1, 'xWR1443 real mode sampling rate must be < 12.5 Msps'];
                } else {
                    if (mmWaveAdvanced.Output.chirpParam_samplingRate > 6.25) return [1, 'xWR1443 complex mode sampling rate must be < 6.25 Msps'];
                }
            }
            else if (mmWaveAdvanced.Output.chirpParam_samplingRate <= 0) return [1, 'real mode sampling rate must be > 0 Msps'];
            return [0, ''];
        },
        // Warn on the sampling rate lower bound
        check_samplingRate_lower: function(){
            return (mmWaveAdvanced.Output.chirpParam_samplingRate < 2) ? [1, 'samplingRate recommended to be at least 2 Msps'] : [0, ''];
        },
        // Warn usage of real
        check_adcOutputFmt: function(){
            return (mmWaveAdvanced.Output.adcbufCfg_adcOutputFmt === 1) ? [1, 'It is generally recommended to use Complex 1x sampling'] : [0, ''];
        },
        // Verify a valid start frequency for the device bandwidth
        check_startFrequency: function(){
            if (mmWaveAdvanced.Input.device === 'xWR6843'){
                if (mmWaveAdvanced.Output.chirpParam_startFrequency > 64 || mmWaveAdvanced.Output.chirpParam_startFrequency < 57) return [1, 'startFrequency must be between 57-64 for a 60GHz device'];
            }
            else if (mmWaveAdvanced.Input.device === 'xWR1843' || mmWaveAdvanced.Input.device === 'xWR1642' || mmWaveAdvanced.Input.device === 'xWR1443'){
                if (mmWaveAdvanced.Output.chirpParam_startFrequency > 81 || mmWaveAdvanced.Output.chirpParam_startFrequency < 73) return [1, 'startFrequency must be between 73-81 for a 77GHz device'];
            }
            return [0, ''];
        },
        // Verify the upper bound of maxBeatFrequency
        check_maxBeatFrequency: function(){
            if (mmWaveAdvanced.Input.device === 'xWR6843'){
                if (mmWaveAdvanced.Output.infoOnly_maxBeatFrequency > 10) {
                    Util.Html.highlightPropWithPrecedence(templateObj.$.ti_widget_container_cc_digOutSampleRate,1, 'conditional');
                    return [1, 'xWR6843 maxBeatFrequency must be < 10: lower digOutSamplingRate'];
                }
            }
            else if (mmWaveAdvanced.Input.device === 'xWR1843'){
                if (mmWaveAdvanced.Output.infoOnly_maxBeatFrequency > 10) {
                    Util.Html.highlightPropWithPrecedence(templateObj.$.ti_widget_container_cc_digOutSampleRate,1, 'conditional');
                    return [1, 'xWR1843 maxBeatFrequency must be < 10: lower digOutSamplingRate'];
                }
            }
            else if (mmWaveAdvanced.Input.device === 'xWR1642'){
                if (mmWaveAdvanced.Output.infoOnly_maxBeatFrequency > 5) {
                    Util.Html.highlightPropWithPrecedence(templateObj.$.ti_widget_container_cc_digOutSampleRate,1, 'conditional');
                    return [1, 'xWR1642 maxBeatFrequency must be < 5: lower digOutSamplingRate'];
                }
            }
            else if (mmWaveAdvanced.Input.device === 'xWR1443'){
                if (mmWaveAdvanced.Output.infoOnly_maxBeatFrequency > 15) {
                    Util.Html.highlightPropWithPrecedence(templateObj.$.ti_widget_container_cc_digOutSampleRate,1, 'conditional');
                    return [1, 'xWR1443 maxBeatFrequency must be < 15: lower digOutSamplingRate'];
                }
            }
            else if (mmWaveAdvanced.Output.infoOnly_maxBeatFrequency <= 0){
                Util.Html.highlightPropWithPrecedence(templateObj.$.ti_widget_container_cc_digOutSampleRate,1, 'conditional');
                return [1, 'maxBeatFrequency must be > 0: increase digOutSamplingRate'];
            }
            return [0, ''];
        },
        // Verify the upper bound of endFrequency
        check_endFrequency: function(){
            if (mmWaveAdvanced.Input.device === 'xWR6843'){
                if (mmWaveAdvanced.Output.results_endFrequency > 64) return [1, 'endFrequency must be < 64GHz for a 60GHz device'];
            }
            else if (mmWaveAdvanced.Input.device === 'xWR1843' || mmWaveAdvanced.Input.device === 'xWR1642' || mmWaveAdvanced.Input.device === 'xWR1443'){
                if (mmWaveAdvanced.Output.results_endFrequency > 81) return [1, 'endFrequency must be < 81GHz for a 77GHz device'];
            }
            return [0, ''];
        },
        check_idleTime: function(){
            // Not currently implemented, but must be here to remove formatting
            return [0, ''];
        },
        check_maxRange: function(){
            // Add a warning to make sure the user does not expect a completely unreasonable max range
            if (mmWaveAdvanced.Output.results_maxRange > 200) {
                return [1, 'The maximum range for a typical detected object is a theoretical calculation and, in reality, is not likely to be this high for this device due to limited transmit power.']
            }
            return [0,''];
        }
    },
    /**
     * An object holding all the error/condition checking functions for SDK5. 
     * If the condition is triggered the function returns a list [1, 'error text'];
     * otherwise the function returns [0, '']
     * 
     * Applies to devices [xWRL6432, xWRL1432]
     */
    SDK5_Rules: {
        // Verify that the rxChannel is not too high for the selected device
        check_channelCfg_rxChannelEn: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.channelCfg5_rxChannelEn > 7){
                    return [1,'rxChannelEn bitmask is trying to use more rx channels than the device has, the max is 0b111 = 7'];
                }
                else if (mmWaveAdvanced.Input.channelCfg5_rxChannelEn <= 0){
                    return [1,'rxChannelEn bitmask is trying to use no rx channels, the min is 0b001 = 1'];
                }
            }
            return [0, ''];
        },
        // Verify that the txChannel is not too high for the selected device
        check_channelCfg_txChannelEn: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.channelCfg5_txChannelEn > 3){
                    return [1,'txChannelEn bitmask is trying to use more tx channels than the device has, the max is 0b11 = 3'];
                }
                else if (mmWaveAdvanced.Input.channelCfg5_txChannelEn <= 0){
                    return [1,'txChannelEn bitmask is trying to use no tx channels, the min is 0b01 = 1'];
                }
            } 
            return [0, ''];
        },
        // Warn on the numSamples lower bound
        check_numSamples: function(){
            return (mmWaveAdvanced.Output.chirpParam_numSamples < 64)?[1, 'Suggested to have number of samples >= 64'] : [0, ''];
        },
        // Warn on the frequencySlope upper bound
        check_frequencySlope: function(){
            if (mmWaveAdvanced.Output.chirpParam_frequencySlope >= 250) {
                return [1, 'Suggested to have slope < 250'];
            }
            else if (mmWaveAdvanced.Output.chirpParam_frequencySlope <= 0){
                return [1, 'Suggested to have slope > 0'];
            }
            return [0, ''];
        },
        // Verify the validSweepBandwidth upper bound
        check_validSweepBandwidth: function(){
            if (mmWaveAdvanced.Output.infoOnly_validSweepBandwidth >= 7000) {
                ['container_cc_chirpComnCfg_numOfAdcSamples',
                 'container_cc_chirpComnCfg_digOutputSampRate',
                 'container_cc_chirpTimingCfg_chirpRfFreqSlope'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget],1, 'conditional');
                });
                return [1, 'validSweepBandwidth too high (>= 7000): reduce bandwidth by lowering freqSlope, lowering numAdcSamples, and/or increasing digOutSampleRate'];
            }
            else if (mmWaveAdvanced.Output.infoOnly_validSweepBandwidth <= 0){
                ['container_cc_chirpComnCfg_numOfAdcSamples',
                 'container_cc_chirpComnCfg_digOutputSampRate',
                 'container_cc_chirpTimingCfg_chirpRfFreqSlope'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget],1, 'conditional');
                });
                return [1, 'validSweepBandwidth too low (<= 0): increase bandwidth by raising freqSlope, raising numAdcSamples, and/or decreasing digOutSampleRate'];
            }
            return [0, ''];
        },
        // Verify the radar cube size will fit into the device's L3 memory
        check_radarCubeSize: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Output.results_radarCubeSize > 512) {
                    ['container_cc_frameCfg5_numOfBurstsInFrame',
                     'container_cc_chirpComnCfg_numOfAdcSamples'].forEach(widget => {
                        Util.Html.highlightPropWithPrecedence(templateObj.$[widget],1, 'conditional');
                    });
                    return [1, 'radarCubeSize must be < xWRL6432/1432 L3 size (512k): Reduce by lowering number of bursts and/or numADCSamples'];
                }
                else if (mmWaveAdvanced.Output.results_radarCubeSize <= 0){
                    ['container_cc_frameCfg5_numOfBurstsInFrame',
                     'container_cc_chirpComnCfg_numOfAdcSamples'].forEach(widget => {
                        Util.Html.highlightPropWithPrecedence(templateObj.$[widget],1, 'conditional');
                    });
                    return [1, 'radarCubeSize must be > 0: Increase by raising number of bursts and/or numADCSamples'];
                }
            }
            return [0, ''];
        },
        // Verify the sampling rate against the ADC mode and the device selected
        check_samplingRate_upper: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){ // Real only device
                if (mmWaveAdvanced.Output.chirpParam_samplingRate > 12.5) return [1, 'xWRL6432/1432 real mode sampling rate must be < 12.5 Msps'];
                else if (mmWaveAdvanced.Output.chirpParam_samplingRate <= 0) return [1, 'real mode sampling rate must be > 0'];
            }
            return [0, ''];
        },
        // Warn on the sampling rate lower bound
        check_samplingRate_lower: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){ // Real only device
                if (mmWaveAdvanced.Output.chirpParam_samplingRate < 2) return [1, 'real mode sampling rate recommended to be at least 2 Msps'];
            }
            return [0, ''];
        },
        // Verify a valid start frequency for the device bandwidth
        check_startFrequency: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432'){
                if (mmWaveAdvanced.Output.chirpParam_startFrequency > 64 || mmWaveAdvanced.Output.chirpParam_startFrequency < 57) return [1, 'startFrequency must be between 57-64 for a 60GHz device'];
            }
            else if (mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Output.chirpParam_startFrequency > 81 || mmWaveAdvanced.Output.chirpParam_startFrequency < 76) return [1, 'startFrequency must be between 76-81 for a 77GHz device'];
            }
            return [0, ''];
        },
        // Verify the upper bound of maxBeatFrequency
        check_maxBeatFrequency: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Output.infoOnly_maxBeatFrequency > 5) {
                    Util.Html.highlightPropWithPrecedence(templateObj.$.container_cc_chirpComnCfg_digOutputSampRate,1,'conditional');
                    return [1, 'xWRL6432/1432 maxBeatFrequency must be < 5: lower digOutSamplingRate'];
                }
                else if (mmWaveAdvanced.Output.infoOnly_maxBeatFrequency <= 0){
                    Util.Html.highlightPropWithPrecedence(templateObj.$.container_cc_chirpComnCfg_digOutputSampRate,1,'conditional');
                    return [1, 'maxBeatFrequency must be > 0: increase digOutSamplingRate'];
                }
            }
            return [0, ''];
        },
        // Verify the upper bound of endFrequency
        check_endFrequency: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432'){
                if (mmWaveAdvanced.Output.results_endFrequency > 64) return [1, 'endFrequency must be < 64GHz for a 60GHz device'];
            }
            else if (mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Output.results_endFrequency > 81) return [1, 'endFrequency must be < 81Hz for a 77GHz device'];
            }
            return [0, ''];
        },
        // Verify the value in chirpTxMimoPatSel 
        check_chirpTxMimoPatSel: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.chirpComnCfg_ChirpTxMimoPatSel != 1 && mmWaveAdvanced.Input.chirpComnCfg_ChirpTxMimoPatSel != 4) {
                    return [1, 'chirpTxMimoPatSel is used to configure BPM or TDM mode. Must be set to either 1 to enable TDM mode and 4 to enable BPM mode. Suggested to set 4.'];
                }
            }
            return [0, ''];
        },
        // Verify the rampEndTime condition, SDK 5
        check5_rampEndTime: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.chirpComnCfg_ChirpRampEndTime < (mmWaveAdvanced.Input.chirpTimingCfg_ChirpAdcSkipSamples/mmWaveAdvanced.Output.chirpParam_samplingRate + mmWaveAdvanced.Output.infoOnly_chirpTime)) {
                    ['container_cc_chirpTimingCfg_chirpAdcSkipSamples',
                     'container_cc_chirpComnCfg_digOutputSampRate',
                     'ti_widget_container_chirpTime'].forEach(widget => {
                        Util.Html.highlightPropWithPrecedence(templateObj.$[widget],1, 'conditional');
                    });
                    return [1, 'rampEndTime must be greater than the sum of the (ADC Skip Samples / Sampling Rate) and chirpTime'];
                }
            }
            return [0, ''];
        },
        //check input for sdk 5 digoutsamprate
        check5_digOutSampleRate: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.chirpComnCfg_DigOutputSampRate < 8 || mmWaveAdvanced.Input.chirpComnCfg_DigOutputSampRate > 100) return [1, 'digOutputSampRate must be between 8 and 100'];
            }
            return [0, ''];
        },
        check5_digOutputBitsSel: function() {
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.chirpComnCfg_DigOutputBitsSel < 0 || mmWaveAdvanced.Input.chirpComnCfg_DigOutputBitsSel > 5) return [1, 'digOutputBitsSel must be between 0 and 5'];
            }
            return [0, ''];
        },
        check5_dfeFirSel: function() {
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.chirpComnCfg_DfeFirSel < 0 || mmWaveAdvanced.Input.chirpComnCfg_DfeFirSel > 1) return [1, 'dfeFirSel must be between 0 and 1'];
            }
            return [0, ''];
        },
        check5_chirpRxHpfSel: function() {
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.chirpComnCfg_ChirpRxHpfSel < 0 || mmWaveAdvanced.Input.chirpComnCfg_ChirpRxHpfSel > 3) return [1, 'chirpRxHpfSel must be between 0 and 3'];
            }
            return [0, ''];
        },
        check5_chirpAdcSkipSamples: function() {
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.chirpTimingCfg_ChirpAdcSkipSamples < 0 || mmWaveAdvanced.Input.chirpTimingCfg_ChirpAdcSkipSamples > 63) return [1, 'chirpAdcSkipSamples must be between 0 and 63'];
            }
            return [0, ''];
        },
        check5_chirpRfFreqSlope: function() {
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.chripTimingCfg_ChirpRfFreqSlope < -399 || mmWaveAdvanced.Input.chripTimingCfg_ChirpRfFreqSlope > 399) return [1, 'chirpRfFreqSlope must be between +/-399'];
            }
            return [0, ''];
        },
        check5_chirpRfFreqStart: function() {
            if (mmWaveAdvanced.Input.device === 'xWRL6432'){
                if (mmWaveAdvanced.Input.chripTimingCfg_ChirpRfFreqStart < 57 || mmWaveAdvanced.Input.chripTimingCfg_ChirpRfFreqStart > 64) return [1, 'chirpRfFreqStart must be between 57 and 64'];
            }
            else if (mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.chripTimingCfg_ChirpRfFreqStart < 76 || mmWaveAdvanced.Input.chripTimingCfg_ChirpRfFreqStart > 81) return [1, 'chirpRfFreqStart must be between 76 and 81'];
            }
            return [0, ''];
        },
        check5_miscCtrl: function() {
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.channelCfg5_MiscCtrl != 0) return [1, 'miscCtrl must be 0 in the current SDK5 version'];
            }
            return [0, ''];
        },
        check5_numOfChirpsInBurst: function() {
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.frameCfg5_NumOfChirpsInBurst < 1 || mmWaveAdvanced.Input.frameCfg5_NumOfChirpsInBurst > 65535) return [1, 'numOfChirpsInBurst must be between 1 and 65535'];
            }
            return [0, ''];
        },
        check5_numOfChirpsAccum: function() {
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.frameCfg5_NumOfChirpsAccum < 0 || mmWaveAdvanced.Input.frameCfg5_NumOfChirpsAccum > 64) return [1, 'numOfChirpsAccum must be between 0 and 64'];
            }
            return [0, ''];
        },
        check5_numOfBurstsInFrame: function() {
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.frameCfg5_NumOfBurstsInFrame < 1 || mmWaveAdvanced.Input.frameCfg5_NumOfBurstsInFrame > 4096) return [1, 'numOfBurstsInFrame must be between 1 and 4096'];
            }
            return [0, ''];
        },
        check5_numOfFrames: function() {
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.frameCfg5_NumOfFrames < 0 || mmWaveAdvanced.Input.frameCfg5_NumOfFrames > 65535) return [1, 'numOfFrames must be between 0 and 65535'];
            }
            return [0, ''];
        },
        //check input for sdk 5 digoutsamprate
        check5_numOfAdcSamples: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.chirpComnCfg_NumOfAdcSamples < 2 || mmWaveAdvanced.Input.chirpComnCfg_NumOfAdcSamples > 2048) return [1, 'numOfAdcSamples must be between 2 to 2048'];
            }
            return [0, ''];
        },
        //check input for sdk 5 burst period
        check5_burstPeriod: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                // This is the old burst period check, which is more relaxed than the new burst idle time check
                /*
                let burstPeriodicity_us = mmWaveAdvanced.Input.frameCfg5_BurstPeriodicity;
                let active_chirp_time_us = mmWaveAdvanced.Output.infoOnly_activeChirpingTime * 1000;
                if (burstPeriodicity_us < active_chirp_time_us) {
                    return [1, 'Burst period, currently ' + burstPeriodicity_us + 'µs must be longer than active chirping time, currently ' + Util.Format.toPrecision(active_chirp_time_us, 3) + 'µs'];
                }
                */
                // Add the burst idle time check
                let burstPeriodicity_us = mmWaveAdvanced.Input.frameCfg5_BurstPeriodicity;
                let numChirpLoops = mmWaveAdvanced.Input.frameCfg5_NumOfChirpsInBurst;
                let idleTime = mmWaveAdvanced.Output.chirpParam_idleTime;
                let rampEndTime = mmWaveAdvanced.Output.chirpParam_rampEndTime;
                let chirpsAccum = mmWaveAdvanced.Input.frameCfg5_NumOfChirpsAccum;
                let burstIdleTime = burstPeriodicity_us - (numChirpLoops * (idleTime + rampEndTime) * chirpsAccum);
                if (burstIdleTime < 115) {
                    ['container_cc_frameCfg5_numOfChirpsInBurst',
                     'container_cc_frameCfg5_numOfChirpsAccum',
                     'container_cc_chirpTimingCfg_chirpIdleTime',
                     'container_cc_chirpComnCfg_chirpRampEndTime'].forEach(widget => {
                        Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                    });
                    return [1, ' With the xWRL6432/1432, burst idle time, which equals burst periodicity - numOfChirpsinBurst(idleTime+rampEndTime) * numOfChirpsAccum), must be greater than 115µs. Currently it is ' + 
                                burstIdleTime  + 'µs. Increase burstPeriodicity, or decrease numOfChirpsinBurst, numOfChirpsAccum, idleTime, or rampEndTime'];
                }

            }
            return [0, ''];
        },
        //check input for sdk 5 frame period
        check5_framePeriod: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                // total bursting time using: mmWaveAdvanced.Input.frameCfg5_BurstPeriodicity*.01*mmWaveAdvanced.Input.frameCfg5_NumOfBurstsInFrame
                //if (mmWaveAdvanced.Input.frameCfg5_FramePeriodicity < (mmWaveAdvanced.Input.frameCfg5_BurstPeriodicity*.001*mmWaveAdvanced.Input.frameCfg5_NumOfBurstsInFrame)) return [1, 'Frame period must be longer than number of bursts * burst period'];
            
                // "The framePeriodicity should be configured to allow a minimum 135 µs inter-frame idle time"
                let FramePeriodicity_ms = mmWaveAdvanced.Input.frameCfg5_FramePeriodicity;
                let FramePeriodicity_us = FramePeriodicity_ms * 1000;   
                let burstPeriodicity_us = mmWaveAdvanced.Input.frameCfg5_BurstPeriodicity;
                let numOfBurstsInFrame = mmWaveAdvanced.Input.frameCfg5_NumOfBurstsInFrame;
                let frameIdleTime_us = FramePeriodicity_us - (burstPeriodicity_us * numOfBurstsInFrame);
                if (frameIdleTime_us < 135) {
                    ['container_cc_frameCfg5_burstPeriodicity',
                    'container_cc_frameCfg5_numOfBurstsInFrame'].forEach(widget => {
                       Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                    });
                    return [1, ' With the xWRL6432/1432, frame idle time, which equals framePeriodicity - (numOfBurstsInFrame * burstPeriodicity), must be greater than 135µs. Currently it is ' + 
                    frameIdleTime_us  + 'µs. Increase framePeriodicity, or decrease numOfBurstsInFrame, or burstPeriodicity'];
                }
            
            }
            return [0, ''];
        },
        check5_idleTime: function(){
            // The xWRL6432/1432 minimum chirpIdleTime cannot be lower than maximum of (4µs - chirpTxStartTime, 3.1µs)
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                let minIdleTime = Math.max(4 - mmWaveAdvanced.Input.chripTimingCfg_ChirpTxStartTime, 3.1);
                if (mmWaveAdvanced.Input.chripTimingCfg_ChirpIdleTime < minIdleTime) {
                    return [1, 'The minimum chirpIdleTime cannot be lower than maximum of (4µs - chirpTxStartTime, 3.1µs) = ' + minIdleTime];
                }
            }
            return [0, ''];
        },
        check5_idleTimeLowPower: function(){
            // When using xWRL6432 in Low power mode, configured by lowPowerCfg command, 
            // the minimum chirpIdleTime cannot be lower than maximum of (6µs - chirpTxStartTime, 3.1µs)
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                let minIdleTime = Math.max(6 - mmWaveAdvanced.Input.chripTimingCfg_ChirpTxStartTime, 3.1);
                if (mmWaveAdvanced.Input.chripTimingCfg_ChirpIdleTime < minIdleTime) {
                    return [1, 'When using xWRL6432/1432 in low power mode (configured by lowPowerCfg command), the minimum chirpIdleTime cannot be lower than maximum of (6µs - chirpTxStartTime, 3.1µs) = ' + minIdleTime];
                }
            }
            return [0, ''];
        },
        check5_maxRange: function(){
            // Add a warning to make sure the user does not expect a completely unreasonable max range
            if ((mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432') && mmWaveAdvanced.Output.results_maxRange > 40) {
                return [1, 'The maximum range for a typical detected object is a theoretical calculation and, in reality, is not likely to be this high for this device due to limited transmit power.'];
            }
            return [0,''];
        },
        check5_adcSaturationAOP: function(){
            // Add a warning to indicate their ADC saturation is too high and will cause detection issues with AOP ONLY
            // The equation is  Rx gain in dB - Tx BO in dB - 6*c_ChirpRxHpfSel + 20*log10 of (freq slope/ 100) 
            
            let adcSaturation = mmWaveAdvanced.Input.factoryCalibCfg_RxGain - mmWaveAdvanced.Input.factoryCalibCfg_TxBackoff - (6 * mmWaveAdvanced.Input.chirpComnCfg_ChirpRxHpfSel ) + 20 * Math.log10(mmWaveAdvanced.Output.chirpParam_frequencySlope / 100);
            
            if (mmWaveAdvanced.Input.chirpComnCfg_ChirpTxMimoPatSel == 1){
                if (mmWaveAdvanced.Input.device === 'xWRL6432' && mmWaveAdvanced.Input.evm === 'xWRL6432AOP' && adcSaturation > 23.5) {
                    return [1, 'ADC Saturation too high. Saturation metric is defined as:\nRX Gain (dB) - TX Backoff (dB) - 6*ChirpRxHpfSel + \n20*log10(Frequency Slope/100Mhzperus)\n\nFor a TDMA configuration, this value is recommended to be below 23.5 to avoid ADC saturation to allow maximum angular and range accuracy\n\nCurrent Value: ' + adcSaturation.toFixed(2) + '\n\nTry a mixture of the following: decrease RX Gain, increase TX Backoff, increase ChirpRxHpfSel, decrease Frequency Slope'];
                }
            }
            
            else if (mmWaveAdvanced.Input.chirpComnCfg_ChirpTxMimoPatSel == 4){
                if (mmWaveAdvanced.Input.device === 'xWRL6432' && mmWaveAdvanced.Input.evm === 'xWRL6432AOP' && adcSaturation > 20.5) {
                    return [1, 'ADC Saturation too high. Saturation metric is defined as:\nRX Gain (dB) - TX Backoff (dB) - 6*ChirpRxHpfSel + \n20*log10(Frequency Slope/100Mhzperus)\n\nFor a BPM configuration, this value is recommended to be below 20.5 to avoid ADC saturation to allow maximum angular and range accuracy\n\nCurrent Value: ' + adcSaturation.toFixed(2) + '\n\nTry a mixture of the following: decrease RX Gain, increase TX Backoff, increase ChirpRxHpfSel, decrease Frequency Slope'];
                }
            }
            return [0,''];
        },
        check5_rxGain: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.factoryCalibCfg_RxGain < 30 || mmWaveAdvanced.Input.factoryCalibCfg_RxGain > 40) return [1, 'RX Gain must be between 30 and 40 dB'];
            }
            return [0, ''];
        },
        check5_txBackoff: function(){
            if (mmWaveAdvanced.Input.device === 'xWRL6432' || mmWaveAdvanced.Input.device === 'xWRL1432'){
                if (mmWaveAdvanced.Input.factoryCalibCfg_TxBackoff < 0 || mmWaveAdvanced.Input.factoryCalibCfg_TxBackoff > 26 || mmWaveAdvanced.Input.factoryCalibCfg_TxBackoff === '') return [1, 'TX Backoff must be between 0 and 26 dB'];
            }
            return [0, ''];
        }
    },
};

let Adv_ErrorMaps = {
    /**
     * A mapping to tie the general front end widgets to their error-checking function
     */
    General_Err: [
        [['ti_widget_container_device'],                Adv_Error.General_Rules.check_device],
        [['ti_widget_container_evmBoard'],              Adv_Error.General_Rules.check_evmBoard],
        [['ti_widget_container_adcMode'],               Adv_Error.General_Rules.check_adcMode] 
    ],
    /**
     * A mapping to tie the SDK3 front end widgets to their error-checking function
     */
    SDK3_Err: [
        [['ti_widget_container_rxChannelEn',
          'container_Adv_output_numRx'],                Adv_Error.SDK3_Rules.check_channelCfg_rxChannelEn],  
        [['ti_widget_container_txChannelEn',
          'container_Adv_output_numTx'],                Adv_Error.SDK3_Rules.check_channelCfg_txChannelEn],  
        [['ti_widget_container_chirpCfg_1'],            Adv_Error.SDK3_Rules.check_chirpCfg1],               
        [['ti_widget_container_chirpCfg_2'],            Adv_Error.SDK3_Rules.check_chirpCfg2],               
        [['ti_widget_container_chirpCfg_3'],            Adv_Error.SDK3_Rules.check_chirpCfg3],             
        [['ti_widget_container_validSweepBandwidth'],   Adv_Error.SDK3_Rules.check_validSweepBandwidth],     
        [['ti_widget_container_rampEndTime',
          'ti_widget_container_cc_rampEndTime'],        Adv_Error.SDK3_Rules.check_rampEndTime],             
        [['ti_widget_container_radarCubeSize'],         Adv_Error.SDK3_Rules.check_radarCubeSize],           
        [['ti_widget_container_samplingRate',
          'ti_widget_container_cc_digOutSampleRate',
          'ti_widget_container_samplingRate'],          Adv_Error.SDK3_Rules.check_samplingRate_upper],      
        [['ti_widget_container_startFrequency',
          'ti_widget_container_cc_startFreq'],          Adv_Error.SDK3_Rules.check_startFrequency],          
        [['ti_widget_container_maxBeatFrequency'],      Adv_Error.SDK3_Rules.check_maxBeatFrequency],        
        [['ti_widget_container_endFrequency'],          Adv_Error.SDK3_Rules.check_endFrequency], 
        [['ti_widget_container_idleTime'],              Adv_Error.SDK3_Rules.check_idleTime],          
    ],
    /**
     * A mapping to tie the SDK5 front end widgets to their error-checking function
     * 
     * If you put an element here to highlight that is in the intermediate side of the tool, that is to say it exists on the 
     * SDK3 and SDK5 side (any element starting with ti_widget_... below) must also be in the SDK3_Err, otherwise its 
     * highlighting won't be peeled back
     */
    SDK5_Err: [
        [['container_cc_channelCfg5_rxChannelEn',
          'container_Adv_output_numRx'],                    Adv_Error.SDK5_Rules.check_channelCfg_rxChannelEn],  
        [['container_cc_channelCfg5_txChannelEn',
          'container_Adv_output_numTx'],                    Adv_Error.SDK5_Rules.check_channelCfg_txChannelEn],  
        [['ti_widget_container_validSweepBandwidth'],       Adv_Error.SDK5_Rules.check_validSweepBandwidth],     
        [['ti_widget_container_radarCubeSize'],             Adv_Error.SDK5_Rules.check_radarCubeSize],           
        [['ti_widget_container_samplingRate',
          'container_cc_chirpComnCfg_digOutputSampRate, '], Adv_Error.SDK5_Rules.check_samplingRate_upper],      
        [['ti_widget_container_startFrequency',
          'container_cc_chirpTimingCfg_chirpRfFreqStart'],  Adv_Error.SDK5_Rules.check_startFrequency],          
        [['ti_widget_container_maxBeatFrequency'],          Adv_Error.SDK5_Rules.check_maxBeatFrequency],        
        [['ti_widget_container_endFrequency'],              Adv_Error.SDK5_Rules.check_endFrequency], 
        [['container_cc_chirpComnCfg_chirpTxMimoPatSel'],   Adv_Error.SDK5_Rules.check_chirpTxMimoPatSel],           
        [['ti_widget_container_rampEndTime',
          'container_cc_chirpComnCfg_chirpRampEndTime'],    Adv_Error.SDK5_Rules.check5_rampEndTime],            
        [['container_cc_chirpComnCfg_digOutputSampRate'],   Adv_Error.SDK5_Rules.check5_digOutSampleRate],
        [['container_cc_chirpComnCfg_digOutputBitsSel'],    Adv_Error.SDK5_Rules.check5_digOutputBitsSel], 
        [['container_cc_chirpComnCfg_dfeFirSel'],           Adv_Error.SDK5_Rules.check5_dfeFirSel],
        [['container_cc_chirpComnCfg_chirpRxHpfSel'],       Adv_Error.SDK5_Rules.check5_chirpRxHpfSel],
        [['container_cc_chirpTimingCfg_chirpAdcSkipSamples'],Adv_Error.SDK5_Rules.check5_chirpAdcSkipSamples],
        [['container_cc_chirpTimingCfg_chirpRfFreqSlope'],  Adv_Error.SDK5_Rules.check5_chirpRfFreqSlope],
        [['container_cc_chirpTimingCfg_chirpRfFreqStart'],  Adv_Error.SDK5_Rules.check5_chirpRfFreqStart],
        [['container_cc_channelCfg5_miscCtrl'],             Adv_Error.SDK5_Rules.check5_miscCtrl],
        [['container_cc_frameCfg5_numOfChirpsInBurst'],     Adv_Error.SDK5_Rules.check5_numOfChirpsInBurst],
        [['container_cc_frameCfg5_numOfChirpsAccum'],       Adv_Error.SDK5_Rules.check5_numOfChirpsAccum],
        [['container_cc_frameCfg5_numOfBurstsInFrame'],     Adv_Error.SDK5_Rules.check5_numOfBurstsInFrame],
        [['container_cc_frameCfg5_numOfFrames'],            Adv_Error.SDK5_Rules.check5_numOfFrames],
        [['container_cc_chirpComnCfg_numOfAdcSamples'],     Adv_Error.SDK5_Rules.check5_numOfAdcSamples],        
        [['container_cc_frameCfg5_burstPeriodicity'],       Adv_Error.SDK5_Rules.check5_burstPeriod], 
        [['container_cc_frameCfg5_framePeriodicity'],       Adv_Error.SDK5_Rules.check5_framePeriod],
        [['ti_widget_container_idleTime',
            'container_cc_chirpTimingCfg_chirpIdleTime'],     Adv_Error.SDK5_Rules.check5_idleTime],
        [['container_cc_factoryCalibCfg_rxGain'],          Adv_Error.SDK5_Rules.check5_rxGain],
        [['container_cc_factoryCalibCfg_txBackoff'],          Adv_Error.SDK5_Rules.check5_txBackoff],
        
    ],
    /**
     * A mapping to tie the SDK3 front end widgets to their warning-checking function
     */
    SDK3_Warn: [
        [['ti_widget_container_numSamples',
          'ti_widget_container_cc_numSamples'],          Adv_Error.SDK3_Rules.check_numSamples],              
        [['ti_widget_container_frequencySlope',
          'ti_widget_container_cc_freqSlope'],           Adv_Error.SDK3_Rules.check_frequencySlope],          
        [['ti_widget_container_samplingRate',
          'ti_widget_container_cc_digOutSampleRate'],    Adv_Error.SDK3_Rules.check_samplingRate_lower],   
        [['ti_widget_container_cc_adcOutputFmt'],        Adv_Error.SDK3_Rules.check_adcOutputFmt],
        [['ti_widget_container_maxRange'],               Adv_Error.SDK3_Rules.check_maxRange]   
    ],
    /**
     * A mapping to tie the SDK5 front end widgets to their warning-checking function
     */
    SDK5_Warn: [
        [['ti_widget_container_numSamples',
          'container_cc_chirpComnCfg_numOfAdcSamples'],     Adv_Error.SDK5_Rules.check_numSamples],              
        [['ti_widget_container_frequencySlope',
          'container_cc_chirpTimingCfg_chirpRfFreqSlope'],  Adv_Error.SDK5_Rules.check_frequencySlope],          
        [['ti_widget_container_samplingRate',
          'container_cc_chirpComnCfg_digOutputSampRate, '], Adv_Error.SDK5_Rules.check_samplingRate_lower],
        [['ti_widget_container_idleTime',
          'container_cc_chirpTimingCfg_chirpIdleTime'],     Adv_Error.SDK5_Rules.check5_idleTimeLowPower],
        [['ti_widget_container_maxRange'],                  Adv_Error.SDK5_Rules.check5_maxRange],
        [['container_cc_factoryCalibCfg_rxGain',
        'container_cc_factoryCalibCfg_txBackoff',
        'container_cc_chirpComnCfg_chirpRxHpfSel',
        'container_cc_chirpTimingCfg_chirpRfFreqSlope'],          Adv_Error.SDK5_Rules.check5_adcSaturationAOP],      
    ],
    /**
     * Cells that are ever highlighted as "related" (conditional)
     */
    Related: [
        'ti_widget_container_cc_numSamples',
        'ti_widget_container_cc_digOutSampleRate',
        'ti_widget_container_cc_freqSlope',
        'ti_widget_container_cc_adcStartTime',
        'ti_widget_container_adcStartTime',
        'ti_widget_container_chirpTime',
        'ti_widget_container_cc_numLoops',
        'ti_widget_container_numLoops',
        'container_cc_frameCfg5_numOfBurstsInFrame',
        'container_cc_chirpComnCfg_numOfAdcSamples',
        'container_cc_chirpComnCfg_digOutputSampRate',
        'container_cc_chirpTimingCfg_chirpRfFreqSlope',
        'container_cc_chirpComnCfg_chirpRampEndTime',
        'container_cc_chirpTimingCfg_chirpAdcSkipSamples',
        'ti_widget_container_cc_adcOutputFmt',
        'container_cc_chirpTimingCfg_chirpIdleTime',
        'container_cc_frameCfg5_numOfChirpsInBurst',
        'container_cc_frameCfg5_numOfChirpsAccum',
        'container_cc_frameCfg5_burstPeriodicity',
        'container_cc_frameCfg5_numOfBurstsInFrame',
    ],
};
