// jshint maxerr:1000
/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ===============================================================================================================
   ===============================================================================================================
                                                     ADVANCED TAB TOOL 
   ===============================================================================================================
   =============================================================================================================== */
let Adv_Globals = {
    Constants: {
        lightSpeed_mps: 299792458,              // speed of light m/us   F46
        kB: 1.38064852e-23,               // Bolzmann constant J/K, kgm^2/s^2K  F47
        cube_4pi: Math.pow(4*Math.PI, 3)  // F48
    },
    /**
     * Supported platforms and their associated EVMs and ADC Modes. 
     * Make sure this matches the dropdown labels/values in GUI Composer exactly.
     * 
     * device: the radar device
     *   |---> evm:  various evms for this device
     *   |---> adcModes: the sampling modes available on the ADC for this device 
     *   |---> sdkVer: the SDK major version that works with this device
     *   |---> maxInterFreq_MHz: low and high boundaries for chirp frequencies [MHz]
     */
    Platforms: {
        'xWR6843':      {
            evms:               ['xWR6843AOP','xWR6843ISK','xWR6843ODS','xWR6843 Horn Antenna'],
            adcModes:           ['Complex 1x','Real'],
            sdkVer:             3,
            maxInterFreq_MHz:   10
        },
        'xWRL6432' :   {
            evms:               ['xWRL6432BOOST', 'xWRL6432AOP'],
            adcModes:           ['Real'],
            sdkVer:             5,
            maxInterFreq_MHz:   5
        },
        'xWRL1432' :   {
            evms:               ['xWRL1432BOOST'],
            adcModes:           ['Real'],
            sdkVer:             5,
            maxInterFreq_MHz:   5
        },
        'xWR1843':      {
            evms:               ['xWR1843AOP','xWR1843BOOST'],
            adcModes:           ['Complex 1x','Real'],
            sdkVer:             3,
            maxInterFreq_MHz:   10
        },
        'xWR1642':      {
            evms:               ['xWR1642BOOST'],
            adcModes:           ['Complex 1x','Real'],
            sdkVer:             3,
            maxInterFreq_MHz:   5
        },
        'xWR1443':      {
            evms:               ['xWR1443BOOST'],
            adcModes:           ['Complex 1x','Real'],
            sdkVer:             3,
            maxInterFreq_MHz:   15
        }
    },
    /**
     * Antenna gains used for each EVM. A todo would be to tuck this into Platforms.
     */
    GainsByEVM: {
        'xWR6843AOP':           5.2,
        'xWR6843ISK':           7,
        'xWR6843ODS':           5,
        'xWR6843 Horn Antenna': 13.1,
        'xWRL6432BOOST':          4,
        'xWRL6432AOP':            4,
        'xWRL1432BOOST':          4,
        'xWR1843AOP':           5.2,
        'xWR1843BOOST':         7,
        'xWR1642BOOST':         10.5,
        'xWR1443BOOST':         9
    },
    /**
     * An object skeleton of every input in the BE mmWaveAdvanced Object
     */
    BackendInputs: {
        // General Inputs:
        'device':                             '', 
        'evm':                                '', 
        'adcMode':                            '', 
        'sdkVer':                             '',
    
        // Configuration Inputs:
        //  |----> profileCfg -----
            'profileCfg_profileID':           '',
            'profileCfg_startFreq':           '',
            'profileCfg_idleTime':            '',
            'profileCfg_adcStartTime':        '',
            'profileCfg_rampEndTime':         '',
            'profileCfg_txOutPowerBackoff':   '',
            'profileCfg_txPhaseShifter':      '',
            'profileCfg_freqSlope':           '',
            'profileCfg_txStartTime':         '',
            'profileCfg_numAdcSamples':       '',
            'profileCfg_digOutSampleRate':    '',
            'profileCfg_hpfCornerFreq1':      '',
            'profileCfg_hpfCornerFreq2':      '',
            'profileCfg_rxGain':              '',

        //  |----> channelCfg -----
            'channelCfg_rxChannelEn':         '',
            'channelCfg_txChannelEn':         '',
            'channelCfg_cascading':           '',

        //  |----> adcbufCfg -----
            'adcbufCfg_subFrameIdx':          '',
            //'adcbufCfg_adcOutputFmt':         '', // Actually treating this as an output from the number format dropdown
            'adcbufCfg_sampleSwap':           '',
            'adcbufCfg_chanInterleave':       '',
            'adcbufCfg_chirpThreshold':       '',

        //  |----> chirpCfg1 -----
            'chirpCfg1_chirpStartIdx':        '',
            'chirpCfg1_chirpEndIdx':          '',
            'chirpCfg1_idleTime':             '',
            'chirpCfg1_profileId':            '',
            'chirpCfg1_startFreqVar':         '',
            'chirpCfg1_freqSlopeVar':         '',
            'chirpCfg1_idleTimeVar':          '',
            'chirpCfg1_adcStartTimeVar':      '',
            'chirpCfg1_txEnableMask':         '',

        //  |----> chirpCfg2 -----
            'chirpCfg2_chirpStartIdx':        '',
            'chirpCfg2_chirpEndIdx':          '',
            'chirpCfg2_idleTime':             '',
            'chirpCfg2_profileId':            '',
            'chirpCfg2_startFreqVar':         '',
            'chirpCfg2_freqSlopeVar':         '',
            'chirpCfg2_idleTimeVar':          '',
            'chirpCfg2_adcStartTimeVar':      '',
            'chirpCfg2_txEnableMask':         '',

        //  |----> chirpCfg3 -----
            'chirpCfg3_chirpStartIdx':        '',
            'chirpCfg3_chirpEndIdx':          '',
            'chirpCfg3_idleTime':             '',
            'chirpCfg3_profileId':            '',
            'chirpCfg3_startFreqVar':         '',
            'chirpCfg3_freqSlopeVar':         '',
            'chirpCfg3_idleTimeVar':          '',
            'chirpCfg3_adcStartTimeVar':      '',
            'chirpCfg3_txEnableMask':         '',

        //  |----> frameCfg -----
            'frameCfg_chirpStartIdx':         '',
            'frameCfg_chirpEndIdx':           '',
            'frameCfg_numLoops':              '',
            'frameCfg_numFrames':             '',
            'frameCfg_framePeriodicity':      '',
            'frameCfg_triggerSelect':         '',
            'frameCfg_frameTriggerDelay':     '',
            
        //SDK 5
        //  |----> chirpComnCfg -----  
            'chirpComnCfg_DigOutputSampRate':      '',
            'chirpComnCfg_DigOutputBitsSel':        '',
            'chirpComnCfg_DfeFirSel':               '',
            'chirpComnCfg_NumOfAdcSamples':         '',
            'chirpComnCfg_ChirpTxMimoPatSel':       '',
            'chirpComnCfg_ChirpRampEndTime':        '',
            'chirpComnCfg_ChirpRxHpfSel':           '',
        //  |----> chirpTimingCfg ----- 
            'chripTimingCfg_ChirpIdleTime':         '',
            'chirpTimingCfg_ChirpAdcSkipSamples':   '',
            'chripTimingCfg_ChirpTxStartTime':      '',
            'chripTimingCfg_ChirpRfFreqSlope':      '',
            'chripTimingCfg_ChirpRfFreqStart':      '',
        //  |----> channelCfg5 -----  
            'channelCfg5_rxChannelEn':              '',
            'channelCfg5_txChannelEn':              '',
            'channelCfg5_MiscCtrl':                 '',
        //  |----> frameCfg5 -----       
            'frameCfg5_NumOfChirpsInBurst':         '',
            'frameCfg5_NumOfChirpsAccum':           '',
            'frameCfg5_BurstPeriodicity':           '',
            'frameCfg5_NumOfBurstsInFrame':         '',
            'frameCfg5_FramePeriodicity':           '',
            'frameCfg5_NumOfFrames':                '',
        
        //  |----> factoryCalibCfg -----       
            'factoryCalibCfg_SaveEnable':          '',
            'factoryCalibCfg_RestoreEnable':       '',
            'factoryCalibCfg_RxGain':               '',
            'factoryCalibCfg_TxBackoff':            '',
            'factoryCalibCfg_FlashOffset':          ''
        
    },
    /**
    * An object skeleton of every output in the BE mmWaveAdvanced Object
    */
    BackendOutputs: {
        //sdk version used by front end (0: SDK3, 1: SDK5)
        'sdkVersion':                  '',
        
        // adcbufCfg
        'adcbufCfg_adcOutputFmt':             '',

        // Chirp Parameters
        'chirpParam_startFrequency':          '',
        'chirpParam_frequencySlope':          '',
        'chirpParam_samplingRate':            '',
        'chirpParam_numSamples':              '',
        'chirpParam_numChirpLoops':           '',
        'chirpParam_idleTime':                '',
        'chirpParam_adcValidStartTime':       '',
        'chirpParam_rampEndTime':             '',
        'chirpParam_framePeriodicity':        '',

        // Device & Board Specific Parameters
        'deviceBoardParam_numRxAntennas':     '',
        'deviceBoardParam_numTxAntennas':     '',
        'deviceBoardParam_numVirtualRx':      '',
        'deviceBoardParam_txGain':            '',
        'deviceBoardParam_rxGain':            '',

        // Information-Only Parameters
        'infoOnly_chirpTime':                 '',
        'infoOnly_maxBeatFrequency':          '',
        'infoOnly_validSweepBandwidth':       '',
        'infoOnly_carrierFrequency':          '',
        'infoOnly_chirpRepitionPeriod':       '',
        'infoOnly_nfftRange':                 '',
        'infoOnly_dutyCycle':                 '',
        'infoOnly_complianceChirpTime':        '',
        'infoOnly_activeChirpingTime':        '',

        // Results
        'results_maxRange':                   '',
        'results_rangeResolution':            '',
        'results_maxVelocity':                '',
        'results_velocityResolution':         '',
        'results_radarCubeSize':              '',
        'results_endFrequency':               ''
    },
    /**
     * A mapping to tie every front-end input widget to its back-end variable
     */
    Inputs: [
        ['ti_widget_dropdown_cc_device',                            'device',                          'dropdown'],
        ['ti_widget_dropdown_cc_evm',                               'evm',                             'dropdown'],
        ['ti_widget_dropdown_cc_adcMode',                           'adcMode',                         'dropdown'],
        ['ti_widget_tabcontainer_sdk',                              'sdkVersion',                      'int'],
        //['ti_widget_textbox_cc_ambTemp',                          'ambTemp',                         'float'],     // never used in calculation
        ['ti_widget_textbox_cc_profileCfg_profileId',               'profileCfg_profileID',            'int'],
        ['ti_widget_textbox_cc_profileCfg_startFreq',               'profileCfg_startFreq',            'float'],
        ['ti_widget_textbox_cc_profileCfg_idleTime',                'profileCfg_idleTime',             'float'],
        ['ti_widget_textbox_cc_profileCfg_adcStartTime',            'profileCfg_adcStartTime',         'float'],
        ['ti_widget_textbox_cc_profileCfg_rampEndTime',             'profileCfg_rampEndTime',          'float'],
        ['ti_widget_textbox_cc_profileCfg_txOutPowerBackoff',       'profileCfg_txOutPowerBackoff',    'int'],
        ['ti_widget_textbox_cc_profileCfg_txPhaseShifter',          'profileCfg_txPhaseShifter',       'int'],
        ['ti_widget_textbox_cc_profileCfg_freqSlope',               'profileCfg_freqSlope',            'float'],
        ['ti_widget_textbox_cc_profileCfg_txStartTime',             'profileCfg_txStartTime',          'float'],
        ['ti_widget_textbox_cc_profileCfg_numAdcSamples',           'profileCfg_numAdcSamples',        'int'],
        ['ti_widget_textbox_cc_profileCfg_digOutSampleRate',        'profileCfg_digOutSampleRate',     'int'],
        ['ti_widget_textbox_cc_profileCfg_hpfCornerFreq_1',         'profileCfg_hpfCornerFreq1',       'int'],
        ['ti_widget_textbox_cc_profileCfg_hpfCornerFreq_2',         'profileCfg_hpfCornerFreq2',       'int'],
        ['ti_widget_textbox_cc_profileCfg_rxGain',                  'profileCfg_rxGain',               'int'],
        ['ti_widget_textbox_cc_channelCfg_rxChannelEn',             'channelCfg_rxChannelEn',          'int'],
        ['ti_widget_textbox_cc_channelCfg_txChannelEn',             'channelCfg_txChannelEn',          'int'],
        ['ti_widget_textbox_cc_channelCfg_cascading',               'channelCfg_cascading',            'int'],
        ['ti_widget_textbox_cc_adcbufCfg_subFrameIdx',              'adcbufCfg_subFrameIdx',           'int'],
        //['ti_widget_textbox_cc_adcbufCfg_adcOutputFmt',           'adcbufCfg_adcOutputFmt',          'int'],   // Now being treated as an output
        ['ti_widget_textbox_cc_adcbufCfg_sampleSwap',               'adcbufCfg_sampleSwap',            'int'],
        ['ti_widget_textbox_cc_adcbufCfg_chanInterleave',           'adcbufCfg_chanInterleave',        'int'],
        ['ti_widget_textbox_cc_adcbufCfg_chirpThreshold',           'adcbufCfg_chirpThreshold',        'int'],
        ['ti_widget_textbox_cc_chirpCfg_1_chirpStartIdx',           'chirpCfg1_chirpStartIdx',         'int'],
        ['ti_widget_textbox_cc_chirpCfg_2_chirpStartIdx',           'chirpCfg2_chirpStartIdx',         'int'],
        ['ti_widget_textbox_cc_chirpCfg_3_chirpStartIdx',           'chirpCfg3_chirpStartIdx',         'int'],
        ['ti_widget_textbox_cc_chirpCfg_1_chirpEndIdx',             'chirpCfg1_chirpEndIdx',           'int'],
        ['ti_widget_textbox_cc_chirpCfg_2_chirpEndIdx',             'chirpCfg2_chirpEndIdx',           'int'],
        ['ti_widget_textbox_cc_chirpCfg_3_chirpEndIdx',             'chirpCfg3_chirpEndIdx',           'int'],
        ['ti_widget_textbox_cc_chirpCfg_1_profileId',               'chirpCfg1_profileId',             'int'],
        ['ti_widget_textbox_cc_chirpCfg_2_profileId',               'chirpCfg2_profileId',             'int'],
        ['ti_widget_textbox_cc_chirpCfg_3_profileId',               'chirpCfg3_profileId',             'int'],
        ['ti_widget_textbox_cc_chirpCfg_1_startFreqVar',            'chirpCfg1_startFreqVar',          'float'],
        ['ti_widget_textbox_cc_chirpCfg_2_startFreqVar',            'chirpCfg2_startFreqVar',          'float'],
        ['ti_widget_textbox_cc_chirpCfg_3_startFreqVar',            'chirpCfg3_startFreqVar',          'float'],
        ['ti_widget_textbox_cc_chirpCfg_1_freqSlopeVar',            'chirpCfg1_freqSlopeVar',          'float'],
        ['ti_widget_textbox_cc_chirpCfg_2_freqSlopeVar',            'chirpCfg2_freqSlopeVar',          'float'],
        ['ti_widget_textbox_cc_chirpCfg_3_freqSlopeVar',            'chirpCfg3_freqSlopeVar',          'float'],
        ['ti_widget_textbox_cc_chirpCfg_1_idleTimeVar',             'chirpCfg1_idleTimeVar',           'float'],
        ['ti_widget_textbox_cc_chirpCfg_2_idleTimeVar',             'chirpCfg2_idleTimeVar',           'float'],
        ['ti_widget_textbox_cc_chirpCfg_3_idleTimeVar',             'chirpCfg3_idleTimeVar',           'float'],
        ['ti_widget_textbox_cc_chirpCfg_1_adcStartTimeVar',         'chirpCfg1_adcStartTimeVar',       'float'],
        ['ti_widget_textbox_cc_chirpCfg_2_adcStartTimeVar',         'chirpCfg2_adcStartTimeVar',       'float'],
        ['ti_widget_textbox_cc_chirpCfg_3_adcStartTimeVar',         'chirpCfg3_adcStartTimeVar',       'float'],
        ['ti_widget_textbox_cc_chirpCfg_1_txEnableMask',            'chirpCfg1_txEnableMask',          'int'],
        ['ti_widget_textbox_cc_chirpCfg_2_txEnableMask',            'chirpCfg2_txEnableMask',          'int'],
        ['ti_widget_textbox_cc_chirpCfg_3_txEnableMask',            'chirpCfg3_txEnableMask',          'int'],
        ['ti_widget_textbox_cc_frameCfg_chirpStartIdx',             'frameCfg_chirpStartIdx',          'int'],
        ['ti_widget_textbox_cc_frameCfg_chirpEndIdx',               'frameCfg_chirpEndIdx',            'int'],
        ['ti_widget_textbox_cc_frameCfg_numLoops',                  'frameCfg_numLoops',               'int'],
        ['ti_widget_textbox_cc_frameCfg_numFrames',                 'frameCfg_numFrames',              'int'],
        ['ti_widget_textbox_cc_frameCfg_framePeriodicity',          'frameCfg_framePeriodicity',       'float'],
        ['ti_widget_textbox_cc_frameCfg_triggerSelect',             'frameCfg_triggerSelect',          'int'],
        ['ti_widget_textbox_cc_frameCfg_frameTriggerDelay',         'frameCfg_frameTriggerDelay',      'float'],
        // SDK 5 INPUTS //
        ['ti_widget_textbox_cc_chirpComnCfg_DigOutputSampRate',     'chirpComnCfg_DigOutputSampRate',       'int'],
        ['ti_widget_textbox_cc_chirpComnCfg_DigOutputBitsSel',      'chirpComnCfg_DigOutputBitsSel',        'int'],
        ['ti_widget_textbox_cc_chirpComnCfg_DfeFirSel',             'chirpComnCfg_DfeFirSel',               'int'],
        /* DFE FIR sel can actually modify the behavior of the LPF, changing the beat frequency equation from .8 or .9 */
        ['ti_widget_textbox_cc_chirpComnCfg_NumOfAdcSamples',       'chirpComnCfg_NumOfAdcSamples',         'int'],
        ['ti_widget_textbox_cc_chirpComnCfg_ChirpTxMimoPatSel',     'chirpComnCfg_ChirpTxMimoPatSel',       'int'],
        ['ti_widget_textbox_cc_chirpComnCfg_ChirpRampEndTime',      'chirpComnCfg_ChirpRampEndTime',        'float'],
        ['ti_widget_textbox_cc_chirpComnCfg_ChirpRxHpfSel',         'chirpComnCfg_ChirpRxHpfSel',           'int'],
        ['ti_widget_textbox_cc_chripTimingCfg_ChirpIdleTime',       'chripTimingCfg_ChirpIdleTime',         'float'],
        ['ti_widget_textbox_cc_chripTimingCfg_ChirpAdcSkipSamples', 'chirpTimingCfg_ChirpAdcSkipSamples',   'int'],
        ['ti_widget_textbox_cc_chripTimingCfg_ChirpTxStartTime',    'chripTimingCfg_ChirpTxStartTime',      'float'],
        ['ti_widget_textbox_cc_chripTimingCfg_ChirpRfFreqSlope',    'chripTimingCfg_ChirpRfFreqSlope',      'float'],
        ['ti_widget_textbox_cc_chripTimingCfg_ChirpRfFreqStart',    'chripTimingCfg_ChirpRfFreqStart',      'float'],
        ['ti_widget_textbox_cc_channelCfg_rxChannelEn_5',           'channelCfg5_rxChannelEn',              'int'],
        ['ti_widget_textbox_cc_channelCfg_txChannelEn_5',           'channelCfg5_txChannelEn',              'int'],
        ['ti_widget_textbox_cc_channelCfg_MiscCtrl',                'channelCfg5_MiscCtrl',                 'int'],
        ['ti_widget_textbox_cc_frameCfg_NumOfChirpsInBurst',        'frameCfg5_NumOfChirpsInBurst',         'int'],
        ['ti_widget_textbox_cc_frameCfg_NumOfChirpsAccum',          'frameCfg5_NumOfChirpsAccum',           'int'],
        ['ti_widget_textbox_cc_frameCfg_BurstPeriodicity',          'frameCfg5_BurstPeriodicity',           'int'],
        ['ti_widget_textbox_cc_frameCfg_NumOfBurstsInFrame',        'frameCfg5_NumOfBurstsInFrame',         'int'],
        ['ti_widget_textbox_cc_frameCfg_FramePeriodicity',          'frameCfg5_FramePeriodicity',           'float'],
        ['ti_widget_textbox_cc_frameCfg_NumOfFrames',               'frameCfg5_NumOfFrames',                'int'],

        ['ti_widget_textbox_cc_factoryCalibCfg_SaveEnable',         'factoryCalibCfg_SaveEnable',           'int'],
        ['ti_widget_textbox_cc_factoryCalibCfg_RestoreEnable',      'factoryCalibCfg_RestoreEnable',        'int'],
        ['ti_widget_textbox_cc_factoryCalibCfg_RxGain',             'factoryCalibCfg_RxGain',               'int'],
        ['ti_widget_textbox_cc_factoryCalibCfg_TxBackoff',          'factoryCalibCfg_TxBackoff',            'int'],
        ['ti_widget_textbox_cc_factoryCalibCfg_FlashOffset',        'factoryCalibCfg_FlashOffset',          'float']
        
    ],
    /**
     * A mapping to tie every front-end intermediate widget to its SDK 3 back-end variable
     */
    SDK3_Intermediates: [
        ['ti_widget_textbox_cc_chirpParam_startFrequency',         'profileCfg_startFreq',          'float', 'ti_widget_textbox_cc_profileCfg_startFreq'],
        ['ti_widget_textbox_cc_chirpParam_frequencySlope',         'profileCfg_freqSlope',          'float', 'ti_widget_textbox_cc_profileCfg_freqSlope'],
        ['ti_widget_textbox_cc_chirpParam_samplingRate',           'profileCfg_digOutSampleRate',   'float', 'ti_widget_textbox_cc_profileCfg_digOutSampleRate'],
        ['ti_widget_textbox_cc_chirpParam_numSamples',             'profileCfg_numAdcSamples',      'int',   'ti_widget_textbox_cc_profileCfg_numAdcSamples'],
        ['ti_widget_textbox_cc_chirpParam_numChirpLoops',          'frameCfg_numLoops',             'int',   'ti_widget_textbox_cc_frameCfg_numLoops'],
        ['ti_widget_textbox_cc_chirpParam_idleTime',               'profileCfg_idleTime',           'float', 'ti_widget_textbox_cc_profileCfg_idleTime'],
        ['ti_widget_textbox_cc_chirpParam_adcValidStartTime',      'profileCfg_adcStartTime',       'float', 'ti_widget_textbox_cc_profileCfg_adcStartTime'],
        ['ti_widget_textbox_cc_chirpParam_rampEndTime',            'profileCfg_rampEndTime',        'float', 'ti_widget_textbox_cc_profileCfg_rampEndTime'],
        ['ti_widget_textbox_cc_chirpParam_framePeriodicity',       'frameCfg_framePeriodicity',     'float', 'ti_widget_textbox_cc_frameCfg_framePeriodicity'],
        ['ti_widget_textbox_cc_deviceBoardParam_numRxAntennas',    'channelCfg_rxChannelEn',        'int',   'ti_widget_textbox_cc_channelCfg_rxChannelEn'],
        ['ti_widget_textbox_cc_deviceBoardParam_numTxAntennas',    'channelCfg_txChannelEn',        'int',   'ti_widget_textbox_cc_channelCfg_txChannelEn']
    ],
    /**
     * A mapping to tie every front-end intermediate widget to its SDK 5 back-end variable
     */
    SDK5_Intermediates: [
        ['ti_widget_textbox_cc_chirpParam_startFrequency',         'chripTimingCfg_ChirpRfFreqStart',   'float', 'ti_widget_textbox_cc_chripTimingCfg_ChirpRfFreqStart'],
        ['ti_widget_textbox_cc_chirpParam_frequencySlope',         'chripTimingCfg_ChirpRfFreqSlope',   'float', 'ti_widget_textbox_cc_chripTimingCfg_ChirpRfFreqSlope'],
        ['ti_widget_textbox_cc_chirpParam_samplingRate',           'chirpComnCfg_DigOutputSampRate',    'float', 'ti_widget_textbox_cc_chirpComnCfg_DigOutputSampRate'],
        ['ti_widget_textbox_cc_chirpParam_numSamples',             'chirpComnCfg_NumOfAdcSamples',      'int',   'ti_widget_textbox_cc_chirpComnCfg_NumOfAdcSamples'],
        ['ti_widget_textbox_cc_chirpParam_numChirpLoops',          'frameCfg5_NumOfBurstsInFrame',      'int',   'ti_widget_textbox_cc_frameCfg_NumOfBurstsInFrame'],
        ['ti_widget_textbox_cc_chirpParam_idleTime',               'chripTimingCfg_ChirpIdleTime',      'float', 'ti_widget_textbox_cc_chripTimingCfg_ChirpIdleTime'],
        ['ti_widget_textbox_cc_chirpParam_adcValidStartTime',      'chirpTimingCfg_ChirpAdcSkipSamples','float', 'ti_widget_textbox_cc_chripTimingCfg_ChirpAdcSkipSamples'],
        ['ti_widget_textbox_cc_chirpParam_rampEndTime',            'chirpComnCfg_ChirpRampEndTime',     'float', 'ti_widget_textbox_cc_chirpComnCfg_ChirpRampEndTime'],
        ['ti_widget_textbox_cc_chirpParam_framePeriodicity',       'frameCfg5_FramePeriodicity',        'float', 'ti_widget_textbox_cc_frameCfg_FramePeriodicity'],
        ['ti_widget_textbox_cc_deviceBoardParam_numRxAntennas',    'channelCfg5_rxChannelEn',           'int',   'ti_widget_textbox_cc_channelCfg_rxChannelEn_5'],
        ['ti_widget_textbox_cc_deviceBoardParam_numTxAntennas',    'channelCfg5_txChannelEn',           'int',   'ti_widget_textbox_cc_channelCfg_txChannelEn_5']
    ],
    /**
     * A mapping to tie every front-end output widget to its back-end variable
     */
    Outputs: [
        ['ti_widget_textbox_cc_chirpParam_startFrequency',         'chirpParam_startFrequency',        'float'],
        ['ti_widget_textbox_cc_chirpParam_frequencySlope',         'chirpParam_frequencySlope',        'float'],
        ['ti_widget_textbox_cc_chirpParam_samplingRate',           'chirpParam_samplingRate',          'float'],
        ['ti_widget_textbox_cc_chirpParam_numSamples',             'chirpParam_numSamples',            'int'],
        ['ti_widget_textbox_cc_chirpParam_numChirpLoops',          'chirpParam_numChirpLoops',         'int'],
        ['ti_widget_textbox_cc_chirpParam_idleTime',               'chirpParam_idleTime',              'float'],
        ['ti_widget_textbox_cc_chirpParam_adcValidStartTime',      'chirpParam_adcValidStartTime',     'float'],
        ['ti_widget_textbox_cc_chirpParam_rampEndTime',            'chirpParam_rampEndTime',           'float'],
        ['ti_widget_textbox_cc_chirpParam_framePeriodicity',       'chirpParam_framePeriodicity',      'float'],
        ['ti_widget_textbox_cc_deviceBoardParam_numRxAntennas',    'deviceBoardParam_numRxAntennas',   'int'],
        ['ti_widget_textbox_cc_deviceBoardParam_numTxAntennas',    'deviceBoardParam_numTxAntennas',   'int'],
        ['ti_widget_textbox_cc_deviceBoardParam_numVirtualRx',     'deviceBoardParam_numVirtualRx',    'int'],
        ['ti_widget_textbox_cc_deviceBoardParam_txGain',           'deviceBoardParam_txGain',          'float'],
        ['ti_widget_textbox_cc_deviceBoardParam_rxGain',           'deviceBoardParam_rxGain',          'float'],
        ['ti_widget_textbox_cc_results_maxRange',                  'results_maxRange',                 'float'],
        ['ti_widget_textbox_cc_results_rangeResolution',           'results_rangeResolution',          'float'],
        ['ti_widget_textbox_cc_results_maxVelocity',               'results_maxVelocity',              'float'],
        ['ti_widget_textbox_cc_results_velocityResolution',        'results_velocityResolution',       'float'],
        ['ti_widget_textbox_cc_results_radarCubeSize',             'results_radarCubeSize',            'float'],
        ['ti_widget_textbox_cc_results_endFrequency',              'results_endFrequency',             'float'],
        ['ti_widget_textbox_cc_infoOnly_chirpTime',                'infoOnly_chirpTime',               'float'],
        ['ti_widget_textbox_cc_infoOnly_maxBeatFrequency',         'infoOnly_maxBeatFrequency',        'float'],
        ['ti_widget_textbox_cc_infoOnly_validSweepBandwidth',      'infoOnly_validSweepBandwidth',     'float'],
        ['ti_widget_textbox_cc_infoOnly_carrierFrequency',         'infoOnly_carrierFrequency',        'float'],
        ['ti_widget_textbox_cc_infoOnly_chirpRepitionPeriod',      'infoOnly_chirpRepitionPeriod',     'float'],
        ['ti_widget_textbox_cc_infoOnly_dutycycle',                'infoOnly_dutyCycle',               'float'],
        ['ti_widget_textbox_cc_infoOnly_complianceChirpTime',      'infoOnly_complianceChirpTime',      'float'],
        ['ti_widget_textbox_cc_infoOnly_activechirpingtime',       'infoOnly_activeChirpingTime',      'float'],
        ['ti_widget_textbox_cc_adcbufCfg_adcOutputFmt',            'adcbufCfg_adcOutputFmt',           'int'],
    ],
};
