// jshint maxerr:1000
/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ===============================================================================================================
   ===============================================================================================================
                            ADVANCED TAB TOOL 
   ===============================================================================================================
   =============================================================================================================== */

   /* ====================== USER INPUT ==========================================================================
 * Take user input and the tool's state and pass it to mmWaveAdvanced (def in advToolBackend.js) for processing
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 * Special-Case Handling: tied to a change event in a specific HTML element
 *      - Adv_onLoadConfig()      
 *          - Description:  load an SDK3 cfg from a local file
 *          - Trigger:      press of LOAD CONFIG button (tied to HTML element in index.gui)
 * 
 *      - Adv_onLoadConfig5()
 *          - Description:  load an SDK5 cfg from a local file
 *          - Trigger:      press of LOAD CONFIG button (tied to HTML element in index.gui)
 *
 *      - Adv_onAutoPopulateButton()
 *          - Description:  populate inputs with the commands entired in the Configuration IO terminal
 *          - Trigger:      press of AUTOPOPULATE button (tied to HTML element in index.gui)
 * 
 *      - Adv_onSaveConfigButton()
 *          - Description:  save a cfg to a local file, wrapper function that calls Adv_onSaveConfig()
 *          - Trigger:      press of SAVE CONFIG button (tied to HTML element in index.gui)
 * 
 *      - Adv_onSaveConfig()
 *          - Description:  save a usable .cfg file to the user's computer
 *          - Trigger:      press of SAVE CONFIG button (tied to HTML element in index.gui)
 * 
 *      - Adv_onSendToPower()
 *          - Description:  send the CLI output to the Power Tool for power consumption estimation 
 *          - Trigger:      press of SEND TO POWER button (tied to HTML element in index.gui)
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 * General-Case Handling: catch and hand-off input into any other input element in the tool (mostly textboxes)
 *      - Adv_onInput(mode)      
 *          - Description:  take input from input cells and pass it to the backend for processing. 
 *                          mode = 'init' used for first time init on loading tool
 *          - Trigger:      input into any non-special-case input
 * 
 *      - Adv_onIntermediateInput()      
 *          - Description:  take input from the FE intermediate side into the BE input side and run update inputs
 *          - Trigger:      input into the intermediate values in Chirp Parameters or Device and Board Specific Parameters
 * ===========================================================================================================
 */
   var Adv_onLoadConfig = function(){
    // myFileLoadDialog is an html input element that exists in index.gui. We are catching the result of it here.
    // At the very bottom of index.gui: "<input type="file" id="myFileLoadDialog" accept=".cfg">"
    gc.File.browseAndLoad(null,null,function(data, fileInfo, err) {
        if(err) {
            alert('Error loading the file!\n\n' + toString(err));
        } else {
            // Place the file into the Configuration I/O Box
            templateObj.$.ti_widget_textbox_autopopulate_input.value = data;
            // Call Autopopulate
            Adv_onAutoPopulate(data, mmWaveAdvanced.Input.device);
        }
    }, myFileLoadDialog);
};
var Adv_onLoadConfig5 = function(){
    // myFileLoadDialog is an html input element that exists in index.gui. We are catching the result of it here.
    // At the very bottom of index.gui: "<input type="file" id="myFileLoadDialog" accept=".cfg">"
    gc.File.browseAndLoad(null,null,function(data, fileInfo, err) {
        if(err) {
            alert('Error loading the file!\n\n' + toString(err));
        } else {
            // Place the file into the Configuration I/O Box
            templateObj.$.ti_widget_textbox_autopopulate_input_5.value = data;
            // Call Autopopulate
            Adv_onAutoPopulate(data, mmWaveAdvanced.Input.device);
        }
    }, myFileLoadDialog);
};
var Adv_onAutoPopulateButton = function(data){
    Adv_onAutoPopulate(data, mmWaveAdvanced.Input.device);
};
var Adv_onSaveConfigButton = function(){
    Adv_onSaveConfig(mmWaveAdvanced.Input.device);
};
var Adv_onSaveConfig = function(device){
    
    if (device != 'xWRL6432' && device != 'xWRL1432'){
        // Take the existing file in the Configuration IO Box, inject our CLI commands back into it from the CLI Commands Box
        var file = templateObj.$.ti_widget_textbox_autopopulate_input.value;
        var editedCLI = templateObj.$.ti_widget_textbox_cliOutput.value;
    }
    else if (device == 'xWRL6432' || device == 'xWRL1432'){
        // Take the existing file in the Configuration IO Box, inject our CLI commands back into it from the CLI Commands Box
        var file = templateObj.$.ti_widget_textbox_autopopulate_input_5.value;
        var editedCLI = templateObj.$.ti_widget_textbox_cliOutput.value;        
    }

    // Verify that there are no errors and that there is text into the Configuration IO Box or CLI commands output
    if (file === '' && editedCLI === ''){
        alert('Insert a file into "Configuration I/O" box by clicking either LOAD CONFIG button and selecting a file or AUTO-POPULATE button after copy and pasting cfg file contents into the box.');
        return;
    }
    
    // If there is text in both Configuration IO Box and CLI, then uptdate Configuration with CLI commands
    if (file !== '' && editedCLI !== ''){
        // Create an array of string by spliting each by newline
        var cliLines = editedCLI.split('\n');
        var lines = file.split('\n');
        if (cliLines[0] === '========== ERRORS ==========') {
            alert('Eliminate the errors before attempting to save the configuration to a file.');
            return;
        }

    var mandatories = [];
        // Verify that all the mandatories have been successfully cleared
        if (mandatories.length !== 0) {
            var errString = 'Not generating a CFG file because you are missing these mandatory commands in the Configuration I/O box:\n';
            mandatories.forEach(mand => errString = errString + mand + '\n');
            alert(errString);
        } else {
            var k = 0;
            var j = 0;
            // Generate the file
            if (device != 'xWRL6432' && device != 'xWRL1432'){
                lines.forEach((line, index, self) => {
                    var elements = line.split(/\s+/);
                    if (elements[0] === 'profileCfg') {
                        cliLines.forEach(cmd => {
                            // Inject our new cmd into the cfg file
                            if (cmd.includes('profileCfg')) {
                                self[index] = cmd;
                            }
                        });
                    }
                    else if (elements[0] === 'channelCfg') {
                        cliLines.forEach(cmd => {
                            // Inject our new cmd into the cfg file
                            if (cmd.includes('channelCfg')) {
                                self[index] = cmd;
                            }
                        });
                    }
                    else if (elements[0] === 'adcbufCfg') {
                        cliLines.forEach(cmd => {
                            // Inject our new cmd into the cfg file
                            if (cmd.includes('adcbufCfg')) {
                                self[index] = cmd;
                            }
                        });
                    }
                    else if (elements[0] === 'chirpCfg') {
                        j = 0;
                        cliLines.forEach(cmd => {
                            // Inject our new cmd into the cfg file
                            if (cmd.includes('chirpCfg')) {
                                if (j === k) {
                                    self[index] = cmd;
                                }
                                j++;
                            }
                        });
                        k++;
                    }
                    else if (elements[0] === 'frameCfg') {
                        cliLines.forEach(cmd => {
                            // Inject our new cmd into the cfg file
                            if (cmd.includes('frameCfg')) {
                                self[index] = cmd;
                            }
                        });
                    }
                });
            }
            else if (device == 'xWRL6432' || device == 'xWRL1432'){
                lines.forEach((line, index, self) => {
                    var elements = line.split(/\s+/);
                    if (elements[0] === 'chirpComnCfg') {
                        cliLines.forEach(cmd => {
                            // Inject our new cmd into the cfg file
                            if (cmd.includes('chirpComnCfg')) {
                                self[index] = cmd;
                            }
                        });
                    }
                    else if (elements[0] === 'chirpTimingCfg') {
                        cliLines.forEach(cmd => {
                            // Inject our new cmd into the cfg file
                            if (cmd.includes('chirpTimingCfg')) {
                                self[index] = cmd;
                            }
                        });
                    }
                    else if (elements[0] === 'channelCfg') {
                        cliLines.forEach(cmd => {
                            // Inject our new cmd into the cfg file
                            if (cmd.includes('channelCfg')) {
                                self[index] = cmd;
                            }
                        });
                    }
                    else if (elements[0] === 'frameCfg') {
                        cliLines.forEach(cmd => {
                            // Inject our new cmd into the cfg file
                            if (cmd.includes('frameCfg')) {
                                self[index] = cmd;
                            }
                        });
                    }
                    else if (elements[0] === 'factoryCalibCfg') {
                        cliLines.forEach(cmd => {
                            // Inject our new cmd into the cfg file
                            if (cmd.includes('factoryCalibCfg')) {
                                self[index] = cmd;
                            }
                        });
                    }
                });            
            }

            // Join the modified lines
            var output = lines.join('\n') + '\n\n';

            // output the file
            gc.File.saveBrowserFile(output, { filename: Util.FileIO.getUniqueFileName('profile','.cfg') }, function (e1) {});
        }
    }
    else if (file == '' && editedCLI !== ''){
        // Create an array of string by spliting each by newline
        var cliLines = editedCLI.split('\n');
        if (cliLines[0] === '========== ERRORS ==========') {
            alert('Eliminate the errors before attempting to save the configuration to a file.');
            return;
        }
        var output = "% This cfg file needs to be completed with SDK commands\n% The commands can be found at <SDK_BASE_DIR/docs/mmwave_sdk_user_guide.pdf\n" + editedCLI;
        // output the file
        gc.File.saveBrowserFile(output, { filename: Util.FileIO.getUniqueFileName('profile','.cfg') }, function (e1) {});
    }
};
var Adv_onAutoPopulate = function(inputCLI, device) {
    var input, lines;
    if (inputCLI) lines = inputCLI;
    else if (device != 'xWRL6432' && device != 'xWRL1432') {
        input = templateObj.$.ti_widget_textbox_autopopulate_input.value;
        lines = input.split('\n');
    }
    else if (device == 'xWRL6432' || device == 'xWRL1432'){
        input = templateObj.$.ti_widget_textbox_autopopulate_input_5.value;
        lines = input.split('\n');
    }
    // Go through each line looking for only the cmds we care about and put those into their own arrays
    var count = 0;
    var changes = {};
    var cli_commands = [];
    console.log(device);
    console.log(lines);
    lines.forEach(line => {
        // Delimit the line by one or more whitespace characters
        var temp = line.split(/\s+/);
        cli_commands.push(temp[0]);
        // check the value of the first line
        if (temp[0] === 'profileCfg') {
            changes['profileCfg_profileID']             = parseInt(     temp[1]    );
            changes['profileCfg_startFreq']             = parseFloat(   temp[2]    );
            changes['profileCfg_idleTime']              = parseFloat(   temp[3]    );
            changes['profileCfg_adcStartTime']          = parseFloat(   temp[4]    );
            changes['profileCfg_rampEndTime']           = parseFloat(   temp[5]    );
            changes['profileCfg_txOutPowerBackoff']     = parseInt(     temp[6]    );
            changes['profileCfg_txPhaseShifter']        = parseInt(     temp[7]    );
            changes['profileCfg_freqSlope']             = parseFloat(   temp[8]    );
            changes['profileCfg_txStartTime']           = parseFloat(   temp[9]    );
            changes['profileCfg_numAdcSamples']         = parseInt(     temp[10]   );
            changes['profileCfg_digOutSampleRate']      = parseInt(     temp[11]   );
            changes['profileCfg_hpfCornerFreq1']        = parseInt(     temp[12]   );
            changes['profileCfg_hpfCornerFreq2']        = parseInt(     temp[13]   );
            changes['profileCfg_rxGain']                = parseInt(     temp[14]   );
        }
        else if (temp[0] === 'chirpComnCfg'){
            changes['chirpComnCfg_DigOutputSampRate']   = parseInt(     temp[1]     );
            changes['chirpComnCfg_DigOutputBitsSel']    = parseInt(     temp[2]     );
            changes['chirpComnCfg_DfeFirSel']           = parseInt(     temp[3]     );
            changes['chirpComnCfg_NumOfAdcSamples']     = parseInt(     temp[4]     );
            changes['chirpComnCfg_ChirpTxMimoPatSel']   = parseInt(     temp[5]      );
            changes['chirpComnCfg_ChirpRampEndTime']    = parseFloat(   temp[6]      );
            changes['chirpComnCfg_ChirpRxHpfSel']       = parseInt(     temp[7]      );
        }   
        else if(temp[0] === 'chirpTimingCfg'){
            changes['chripTimingCfg_ChirpIdleTime']         = parseFloat(  temp[1]      );
            changes['chirpTimingCfg_ChirpAdcSkipSamples']   = parseInt(    temp[2]      );
            changes['chripTimingCfg_ChirpTxStartTime']      = parseFloat(  temp[3]      );
            changes['chripTimingCfg_ChirpRfFreqSlope']      = parseFloat(  temp[4]      );
            changes['chripTimingCfg_ChirpRfFreqStart']      = parseFloat(  temp[5]      );
        }
        else if (temp[0] === 'channelCfg' && (device != 'xWRL6432' && device != 'xWRL1432')) {
            changes['channelCfg_rxChannelEn']           = parseInt(     temp[1]    );
            changes['channelCfg_txChannelEn']           = parseInt(     temp[2]    );
            changes['channelCfg_cascading']             = parseInt(     temp[3]    );
        }
        else if (temp[0] === 'channelCfg' && (device === 'xWRL6432' || device === 'xWRL1432')) {
            changes['channelCfg5_rxChannelEn']           = parseInt(     temp[1]    );
            changes['channelCfg5_txChannelEn']           = parseInt(     temp[2]    );
            changes['channelCfg5_MiscCtrl']              = parseInt(     temp[3]    );
        }
        else if (temp[0] === 'adcbufCfg') {
            changes['adcbufCfg_subFrameIdx']            = parseInt(     temp[1]    );
            changes['adcbufCfg_adcOutputFmt']           = parseInt(     temp[2]    );
            changes['adcbufCfg_sampleSwap']             = parseInt(     temp[3]    );
            changes['adcbufCfg_chanInterleave']         = parseInt(     temp[4]    );
            changes['adcbufCfg_chirpThreshold']         = parseInt(     temp[5]    );
        }
        else if (temp[0] === 'chirpCfg') {
            if (count === 0) {
                changes['chirpCfg1_chirpStartIdx']      = parseInt(     temp[1]    );
                changes['chirpCfg1_chirpEndIdx']        = parseInt(     temp[2]    );
                changes['chirpCfg1_profileId']          = parseInt(     temp[3]    );
                changes['chirpCfg1_startFreqVar']       = parseFloat(   temp[4]    );
                changes['chirpCfg1_freqSlopeVar']       = parseFloat(   temp[5]    );
                changes['chirpCfg1_idleTimeVar']        = parseFloat(   temp[6]    );
                changes['chirpCfg1_adcStartTimeVar']    = parseFloat(   temp[7]    );
                changes['chirpCfg1_txEnableMask']       = parseInt(     temp[8]    ); 
            }
            else if (count === 1) {
                changes['chirpCfg2_chirpStartIdx']      = parseInt(     temp[1]    );
                changes['chirpCfg2_chirpEndIdx']        = parseInt(     temp[2]    );
                changes['chirpCfg2_profileId']          = parseInt(     temp[3]    );
                changes['chirpCfg2_startFreqVar']       = parseFloat(   temp[4]    );
                changes['chirpCfg2_freqSlopeVar']       = parseFloat(   temp[5]    );
                changes['chirpCfg2_idleTimeVar']        = parseFloat(   temp[6]    );
                changes['chirpCfg2_adcStartTimeVar']    = parseFloat(   temp[7]    );
                changes['chirpCfg2_txEnableMask']       = parseInt(     temp[8]    ); 
            }
            else {
                changes['chirpCfg3_chirpStartIdx']      = parseInt(     temp[1]    );
                changes['chirpCfg3_chirpEndIdx']        = parseInt(     temp[2]    );
                changes['chirpCfg3_profileId']          = parseInt(     temp[3]    );
                changes['chirpCfg3_startFreqVar']       = parseFloat(   temp[4]    );
                changes['chirpCfg3_freqSlopeVar']       = parseFloat(   temp[5]    );
                changes['chirpCfg3_idleTimeVar']        = parseFloat(   temp[6]    );
                changes['chirpCfg3_adcStartTimeVar']    = parseFloat(   temp[7]    );
                changes['chirpCfg3_txEnableMask']       = parseInt(     temp[8]    ); 
            }
            count++;
        }
        else if (temp[0] === 'frameCfg' && (device != 'xWRL6432' && device != 'xWRL1432')) {
            changes['frameCfg_chirpStartIdx']           = parseInt(     temp[1]    );
            changes['frameCfg_chirpEndIdx']             = parseInt(     temp[2]    );
            changes['frameCfg_numLoops']                = parseInt(     temp[3]    );
            changes['frameCfg_numFrames']               = parseInt(     temp[4]    );
            changes['frameCfg_framePeriodicity']        = parseFloat(   temp[5]    );
            changes['frameCfg_triggerSelect']           = parseInt(     temp[6]    );
            changes['frameCfg_frameTriggerDelay']       = parseFloat(   temp[7]    );
        }
        else if(temp[0] === 'frameCfg' && (device == 'xWRL6432' || device == 'xWRL1432')){
            changes['frameCfg5_NumOfChirpsInBurst']         = parseInt(     temp[1]      );
            changes['frameCfg5_NumOfChirpsAccum']           = parseInt(     temp[2]      );
            changes['frameCfg5_BurstPeriodicity']           = parseInt(     temp[3]      );
            changes['frameCfg5_NumOfBurstsInFrame']         = parseInt(     temp[4]      );
            changes['frameCfg5_FramePeriodicity']           = parseFloat(   temp[5]      );
            changes['frameCfg5_NumOfFrames']                = parseInt(     temp[6]      );
        }
        else if(temp[0] === 'factoryCalibCfg' && (device == 'xWRL6432' || device == 'xWRL1432')){
            changes['factoryCalibCfg_SaveEnable']         = parseInt(     temp[1]      );
            changes['factoryCalibCfg_RestoreEnable']      = parseInt(     temp[2]      );
            changes['factoryCalibCfg_RxGain']             = parseInt(     temp[3]      );
            changes['factoryCalibCfg_TxBackoff']          = parseInt(     temp[4]      );
            changes['factoryCalibCfg_FlashOffset']        = parseFloat(   temp[5]      );
        }
    });

    // If NaN, delete that key-value pair from changes:
    for (const field in changes) {
        if (isNaN(changes[field]) === true) {
            delete changes[field];
        }
    }

    // Assign to CDT Input:
    for (const field in changes) {
        if (field == 'adcbufCfg_adcOutputFmt') {
            mmWaveAdvanced.Input.adcMode = (changes[field] === 0) ? 'Complex 1x': 'Real';
        }
        else{
            mmWaveAdvanced.Input[field] = changes[field];
        }
    }

    // Update everything
    Adv_onInput('init');
    saveObjCookie();
};
var Adv_onInput = function(mode) {
    // For ease of reading, get the widget context
    var widget = templateObj.$;

    // Reflect mmWaveAdvanced object's input value in the textboxes & dropdowns:
    if (mode === 'init'){
        Adv_Globals.Inputs.forEach(widgetMap => {
            // Reflect the inputs:
            if (widgetMap[2] == 'dropdown') {
                Util.Html.updateDroplist(widget[widgetMap[0]],mmWaveAdvanced.Input[widgetMap[1]]);
            }
            else {
                Util.Html.updateTextbox(widget[widgetMap[0]],mmWaveAdvanced.Input[widgetMap[1]]);
            }
        });
        
        // Pass the changes to the backend
        mmWaveAdvanced.updateInput();
    }
    else {
        // Object to store all changes to the input fields, passed to mmWaveAdvanced.updateInput(...)
        var changes = {}; 
        
        // Loop through every input field and grab its current value from the front-end
        Adv_Globals.Inputs.forEach(widgetMap => {

            // int type input fields
            if (widgetMap[2] == 'int') {
                var new_val = parseInt(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    changes[widgetMap[1]] = new_val;
                }
            }
    
            // float type input fields
            else if (widgetMap[2] == 'float') {
                var new_val = parseFloat(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    changes[widgetMap[1]] = new_val;
                }
            }
    
            // dropdown (string) type input fields
            else {
                var new_val = widget[widgetMap[0]].selectedValue; // String of selected dropdown value
                changes[widgetMap[1]] = new_val;
    
                // if device, update the two dropdowns and sdk version:
                if (widgetMap[1] === 'device') {
                    // Update Dropdown Lists:
                    var evmList     = mmWaveAdvanced.get_allowed_evms(new_val);
                    var adcModeList = mmWaveAdvanced.get_allowed_adcMode(new_val);
                    var sdkVersion  = mmWaveAdvanced.get_supported_SDK(new_val);    // 0 = SDK 3, 1 = SDK 5
                    if (evmList === false || adcModeList === false) {
                        console.log('ERROR: Device not in the mmWaveAdvanced.Platform object');
                    }
                    else{
                        Util.Html.updateDroplistOptions(widget.ti_widget_dropdown_cc_evm,     evmList);
                        Util.Html.updateDroplistOptions(widget.ti_widget_dropdown_cc_adcMode, adcModeList);
                        if (evmList.indexOf(widget.ti_widget_dropdown_cc_evm.selectedValue) < 0){
                            Util.Html.updateDroplist(widget.ti_widget_dropdown_cc_evm, evmList[0]);
                            changes['evm'] = evmList[0];
                        }
                        if (adcModeList.indexOf(widget.ti_widget_dropdown_cc_adcMode.selectedValue) < 0){
                            Util.Html.updateDroplist(widget.ti_widget_dropdown_cc_adcMode, adcModeList[0]);
                            changes['adcMode'] = adcModeList[0];
                        }
                    }
                    // update sdk ver tab
                    Util.Html.selectTab(widget.ti_widget_tabcontainer_sdk, sdkVersion);
                }
            }
        });
        // Pass the changes to the backend
        mmWaveAdvanced.updateInput(changes);
    }

    // Update the output fields to reflect the changes made in the input fields
    Adv_updateOutput();
};
var Adv_onIntermediateInput = function(){
    var widget = templateObj.$;
    var changes = {};
    var sdkVersion  = mmWaveAdvanced.get_supported_SDK(mmWaveAdvanced.Input.device);
    // If SDK 3
    if (sdkVersion == '0') {
        Adv_Globals.SDK3_Intermediates.forEach(widgetMap => {
            // Specifically handle a chnage in sampling rate
            if (widgetMap[0] === 'ti_widget_textbox_cc_chirpParam_samplingRate') {
                var new_val = parseFloat(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    changes[widgetMap[1]] = parseInt(new_val * 1000); // Shift units in reverse
                    Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                }
            }
            // Specifically handle a change in the Number of Rx/Tx fields since they are not mapped one-to-one
            else if (widgetMap[0] === 'ti_widget_textbox_cc_deviceBoardParam_numRxAntennas'){
                var new_val = parseInt(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    if (new_val >= 4)       changes[widgetMap[1]] = 15;
                    else if (new_val == 3)  changes[widgetMap[1]] = 7;
                    else if (new_val == 2)  changes[widgetMap[1]] = 3;
                    else                    changes[widgetMap[1]] = 1;
                    Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                }
            } 
            else if (widgetMap[0] === 'ti_widget_textbox_cc_deviceBoardParam_numTxAntennas'){
                var new_val = parseInt(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    if (mmWaveAdvanced.Input.device == 'xWR1642') {
                        if (new_val >= 2)   changes[widgetMap[1]] = 3;
                        else                changes[widgetMap[1]] = 1;
                    }
                    else{
                        if (new_val >= 3)       changes[widgetMap[1]] = 7;
                        else if (new_val == 2)  changes[widgetMap[1]] = 3;
                        else                    changes[widgetMap[1]] = 1;
                    }
                    Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                }
            } 
            // Handle everything else normally
            else {
                // int type input fields
                if (widgetMap[2] == 'int') {
                    var new_val = parseInt(widget[widgetMap[0]].value);
                    if (isNaN(new_val) === false){
                        changes[widgetMap[1]] = new_val;
                        Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                    }
                }
                // float type input fields
                else if (widgetMap[2] == 'float') {
                    var new_val = parseFloat(widget[widgetMap[0]].value);
                    if (isNaN(new_val) === false){
                        changes[widgetMap[1]] = new_val;
                        Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                    }
                }
            }
        });
    }
    // If SDK 5
    else if (sdkVersion == '1') {
        Adv_Globals.SDK5_Intermediates.forEach(widgetMap => {
            // Specifically handle a change in sampling rate
            if (widgetMap[0] === 'ti_widget_textbox_cc_chirpParam_samplingRate') {
                var new_val = parseFloat(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    changes[widgetMap[1]] = Math.round(100/new_val); // Shift units in reverse
                    Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                }
            }
            else if (widgetMap[0] === 'ti_widget_textbox_cc_chirpParam_numChirpLoops') {
                var new_val = parseInt(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    changes[widgetMap[1]] = parseInt(new_val/mmWaveAdvanced.Input.frameCfg5_NumOfChirpsInBurst); // Shift units in reverse
                    Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                }
            }
            else if (widgetMap[0] === 'ti_widget_textbox_cc_chirpParam_adcValidStartTime') {
                var new_val = parseFloat(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    changes[widgetMap[1]] = Math.round(new_val * mmWaveAdvanced.Output.chirpParam_samplingRate); // Shift units in reverse
                    Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                }
            }
            else if (widgetMap[0] === 'ti_widget_textbox_cc_deviceBoardParam_numRxAntennas'){
                var new_val = parseInt(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    if (new_val >= 3)       changes[widgetMap[1]] = 7;
                    else if (new_val == 2)  changes[widgetMap[1]] = 3;
                    else                    changes[widgetMap[1]] = 1;
                    Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                }
            } 
            else if (widgetMap[0] === 'ti_widget_textbox_cc_deviceBoardParam_numTxAntennas'){
                var new_val = parseInt(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    if (new_val >= 2)       changes[widgetMap[1]] = 3;
                    else                    changes[widgetMap[1]] = 1;
                    Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                }
            } 
            // Handle everything else normally
            else {
                // int type input fields
                if (widgetMap[2] == 'int') {
                    var new_val = parseInt(widget[widgetMap[0]].value);
                    if (isNaN(new_val) === false){
                        changes[widgetMap[1]] = new_val;
                        Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                    }
                }
                // float type input fields
                else if (widgetMap[2] == 'float') {
                    var new_val = parseFloat(widget[widgetMap[0]].value);
                    if (isNaN(new_val) === false){
                        changes[widgetMap[1]] = new_val;
                        Util.Html.updateTextbox(widget[widgetMap[3]],changes[widgetMap[1]]);
                    }
                }
            }
        });
    }

    // debug
    console.log(changes);

    // Pass chnages to the backend
    mmWaveAdvanced.updateInput(changes);
    // Update the output fields to reflect the changes made in the input fields
    Adv_updateOutput();
};
// Send the advanced configuration to the power tab
var Adv_onSendToPower = function() {
    var widget = templateObj.$;
    var device = widget.ti_widget_dropdown_cc_device.selectedValue;
    // Only evaluate if the device is xWRL6432 in the drop down for now
    if (device == 'xWRL6432' || device == 'xWRL1432') {
        if(widget.ti_widget_textbox_cliOutput.value.length > 0){
            // reflect the device textbox value in the power tab
            var changes = {};
            changes['device'] = mmWaveAdvanced.Input.device;
            changes['evm'] = mmWaveAdvanced.Input.evm;
            changes['mode'] = "power"; // use power optimized by default
            changes['io'] = "3.3V"; // use 3.3V I/O by default
            changes['proc_time'] = 10;
            changes['num_rx'] = mmWaveAdvanced.Output.deviceBoardParam_numRxAntennas;
            changes['num_tx'] = mmWaveAdvanced.Output.deviceBoardParam_numTxAntennas;
            // if (device == 'xWRL6432') changes['frequency_range'] = '60-64';
            // else if (device == 'xWRL1432') changes['frequency_range'] = '77-81';
            changes['rampendtime'] = mmWaveAdvanced.Input.chirpComnCfg_ChirpRampEndTime;
            changes['chirpidle'] = mmWaveAdvanced.Input.chripTimingCfg_ChirpIdleTime;
            changes['numbursts'] = mmWaveAdvanced.Input.frameCfg5_NumOfBurstsInFrame;
            changes['frametime'] = mmWaveAdvanced.Input.frameCfg5_FramePeriodicity;
            changes['numchirps'] = mmWaveAdvanced.Input.frameCfg5_NumOfChirpsInBurst;
            changes['chirps_accum'] = mmWaveAdvanced.Input.frameCfg5_NumOfChirpsAccum;
            changes['burst_periodicity'] = mmWaveAdvanced.Input.frameCfg5_BurstPeriodicity;
            mmWavePower.updateInput(changes);
            Power_onInput('init');
            Util.Html.selectTab(widget.tool_tabcontainer, 2); // 0 - basic, 1 - advanced, 2 - power
            widget.viewport.scrollTop = 0;
        }
    }
    else {
        widget.ti_widget_textbox_cliOutput.value = 'Only xWRL6432 and xWRL1432 are presently supported in the Power Estimator.\n' + 
                                                    '===============================================================================\n\n' +  
                                                    widget.ti_widget_textbox_cliOutput.value;
    }
};
/* ====================== OUTPUT ==========================================================================
 * Take the processed output data from mmWaveAdvanced (def in advToolBackend.js) and update the output fields
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *      - Adv_updateOutput()      
 *          - Description:  populate the output fields with the processed results in mmWaveAdvanced.Output
 *          - Trigger:      call after Adv_onInput() and mmWaveAdvanced.updateInput() 
 * 
 *      - Adv_updateErrors()
 *          - Description:  based on the tool's state, determined and apply error-checks, create feedback list
 *          - Trigger:      called by Adv_updateOutput()
 *
 *      - Adv_generateCliOutput(feedbackStr)
 *          - Description:  take the feedback list from Adv_updateErrors() and create terminal output
 *          - Trigger:      called by Adv_updateOutput() after Adv_updateErrors()
 * ===========================================================================================================
 */
var Adv_updateOutput = function() {
    // For ease of reading, get the widget context
    var widget = templateObj.$;
    
    // Reflect the updated output values computed in the mmWaveAdvanced backend to the GUI frontend
    Adv_Globals.Outputs.forEach(widgetMap => {
        if (widgetMap[2] === 'float'){
            Util.Html.updateTextbox(widget[widgetMap[0]], mmWaveAdvanced.Output[widgetMap[1]], 3);  // Limit to 3 decimal places
        }
        else{
            Util.Html.updateTextbox(widget[widgetMap[0]], mmWaveAdvanced.Output[widgetMap[1]]);
        }
    });

    // Update the conditional error checking
    var feedback = Adv_updateErrors();
    Adv_generateCliOutput(feedback);
};
var Adv_updateErrors = function() {
    var widget = templateObj.$;
    var sdkVersion  = mmWaveAdvanced.get_supported_SDK(mmWaveAdvanced.Input.device);

    // Strings that hold all the feedback from the error/warning checks
    var errString = '';
    var warnString = '';

    // Remove all error-related cell highlighting then apply it fresh:
    Adv_ErrorMaps.General_Err.forEach(cellName => Util.Html.highlightPropWithPrecedence(widget[cellName],0,'error1'));
    Adv_ErrorMaps.SDK3_Err.forEach(cellName => Util.Html.highlightPropWithPrecedence(widget[cellName],0,'error1'));
    Adv_ErrorMaps.SDK5_Err.forEach(cellName => Util.Html.highlightPropWithPrecedence(widget[cellName],0,'error1'));
    Adv_ErrorMaps.SDK3_Warn.forEach(cellName => Util.Html.highlightPropWithPrecedence(widget[cellName],0,'warning'));
    Adv_ErrorMaps.SDK5_Warn.forEach(cellName => Util.Html.highlightPropWithPrecedence(widget[cellName],0,'warning'));
    Adv_ErrorMaps.Related.forEach(cellName => Util.Html.highlightPropWithPrecedence(widget[cellName],0,'conditional'));


    // Evaluate the general error rules
    Adv_ErrorMaps.General_Err.forEach(map => {
        var widgetNames, isErr, errText;
        widgetNames  = map[0];                  // grab the widgets involved in this error check
        [isErr, errText] = map[1]();            // grab the return value from the error check function call
        if (errText !== '') errString += errText + '\n\n'; 
        widgetNames.forEach(widgetName => Util.Html.highlightPropWithPrecedence(widget[widgetName],isErr,'error1'));
    });

    if (sdkVersion == '0') {
        // Evaluate the SDK3 error rules and apply conditional formatting as necessary
        Adv_ErrorMaps.SDK3_Err.forEach(map => {
            var widgetNames, isErr, errText;
            widgetNames  = map[0];                  // grab the widgets involved in this error check
            [isErr, errText] = map[1]();        // grab the return value from the error check function call
            if (errText !== '') errString += '% ' + errText + '\n\n'; 
            widgetNames.forEach(widgetName => Util.Html.highlightPropWithPrecedence(widget[widgetName],isErr,'error1'));
        });
        // Evaluate the SDK3 warning rules and apply conditional formatting as necessary
        Adv_ErrorMaps.SDK3_Warn.forEach(map => {
            var widgetNames, isErr, warnText;
            widgetNames  = map[0];              // grab the widgets involved in this warning check
            [isErr, warnText] = map[1]();       // grab the return value from the warning check function call
            if (warnText !== '') warnString += '% ' + warnText + '\n\n'; 
            widgetNames.forEach(widgetName => Util.Html.highlightPropWithPrecedence(widget[widgetName],isErr,'warning'));
        });
    }
    else if (sdkVersion == '1') {
        // Evaluate the SDK5 error rules and apply conditional formatting
        Adv_ErrorMaps.SDK5_Err.forEach(map => {
            var widgetNames, isErr, errText;
            widgetNames  = map[0];                  // grab the widgets involved in this error check
            [isErr, errText] = map[1]();        // grab the return value from the error check function call
            if (errText !== '') errString += '% ' + errText + '\n\n'; 
            widgetNames.forEach(widgetName => Util.Html.highlightPropWithPrecedence(widget[widgetName],isErr,'error1'));
        });
        // Evaluate the SDK5 warning rules and apply conditional formatting as necessary
        Adv_ErrorMaps.SDK5_Warn.forEach(map => {
            var widgetNames, isErr, warnText;
            widgetNames  = map[0];              // grab the widgets involved in this warning check
            [isErr, warnText] = map[1]();       // grab the return value from the warning check function call
            if (warnText !== '') warnString += '% ' + warnText + '\n\n'; 
            widgetNames.forEach(widgetName => Util.Html.highlightPropWithPrecedence(widget[widgetName],isErr,'warning'));
        });
    }
    return [errString,warnString];
};
var Adv_generateCliOutput = function(feedbackStr) {
    var widget = templateObj.$;
    var Input = mmWaveAdvanced.Input;
    var Output = mmWaveAdvanced.Output;
    var device = Input.device;
    var cliOutput = '';
    var errString = feedbackStr[0];
    var warnString = feedbackStr[1];

    // If there is no errString contents, cliOutput the cliCmds; otherwise cliOutput the errString
    if (errString === ''){
        if (device != 'xWRL6432' && device != 'xWRL1432'){
            // profileCfg
            if (Input.profileCfg_profileID          !== '' &&
                Input.profileCfg_startFreq          !== '' &&
                Input.profileCfg_idleTime           !== '' &&
                Input.profileCfg_adcStartTime       !== '' &&
                Input.profileCfg_rampEndTime        !== '' &&
                Input.profileCfg_txOutPowerBackoff  !== '' &&
                Input.profileCfg_txPhaseShifter     !== '' &&
                Input.profileCfg_freqSlope          !== '' &&
                Input.profileCfg_txStartTime        !== '' &&
                Input.profileCfg_numAdcSamples      !== '' &&
                Input.profileCfg_digOutSampleRate   !== '' &&
                Input.profileCfg_hpfCornerFreq1     !== '' &&
                Input.profileCfg_hpfCornerFreq2     !== '' &&
                Input.profileCfg_rxGain             !== '') 
            {
                cliOutput =
                    cliOutput + 'profileCfg ' + 
                    Input.profileCfg_profileID          + ' ' +
                    Input.profileCfg_startFreq          + ' ' +
                    Input.profileCfg_idleTime           + ' ' + 
                    Input.profileCfg_adcStartTime       + ' ' +
                    Input.profileCfg_rampEndTime        + ' ' +
                    Input.profileCfg_txOutPowerBackoff  + ' ' +
                    Input.profileCfg_txPhaseShifter     + ' ' +
                    Input.profileCfg_freqSlope          + ' ' +
                    Input.profileCfg_txStartTime        + ' ' +
                    Input.profileCfg_numAdcSamples      + ' ' +
                    Input.profileCfg_digOutSampleRate   + ' ' +
                    Input.profileCfg_hpfCornerFreq1     + ' ' +
                    Input.profileCfg_hpfCornerFreq2     + ' ' +
                    Input.profileCfg_rxGain             + '\n';
            } else {
                cliOutput = cliOutput + 'Make sure to provide values for all fields in profileCfg.\n';
            }
            // channelCfg
            if (Input.channelCfg_rxChannelEn !== '' &&
                Input.channelCfg_txChannelEn !== '' &&
                Input.channelCfg_cascading   !== '') 
            {
                cliOutput = 
                    cliOutput + 'channelCfg ' + 
                    Input.channelCfg_rxChannelEn    + ' ' +
                    Input.channelCfg_txChannelEn    + ' ' +
                    Input.channelCfg_cascading      + '\n';
            } else {
                cliOutput = cliOutput + 'Make sure to provide values for all fields in channelCfg.\n';
            }
            // adcBufCfg
            if (Input.adcbufCfg_subFrameIdx     !== '' &&
                Output.adcbufCfg_adcOutputFmt    !== '' &&
                Input.adcbufCfg_sampleSwap      !== '' &&
                Input.adcbufCfg_chanInterleave  !== '' &&
                Input.adcbufCfg_chirpThreshold  !== '') 
            {
                cliOutput = 
                    cliOutput + 'adcbufCfg ' + 
                    Input.adcbufCfg_subFrameIdx     + ' ' +
                    Output.adcbufCfg_adcOutputFmt   + ' ' +
                    Input.adcbufCfg_sampleSwap      + ' ' +
                    Input.adcbufCfg_chanInterleave  + ' ' +
                    Input.adcbufCfg_chirpThreshold  + '\n';
            } else {
                cliOutput = cliOutput + 'Make sure to provide values for all fields in adcbufCfg.\n';
            }
            // chirpCfg 1
            if (widget.ti_widget_container_chirpCfg_1.style.visibility !== 'hidden' &&
                Input.chirpCfg1_chirpStartIdx       !== '' &&
                Input.chirpCfg1_chirpEndIdx         !== '' &&
                Input.chirpCfg1_profileId           !== '' &&
                Input.chirpCfg1_startFreqVar        !== '' &&
                Input.chirpCfg1_freqSlopeVar        !== '' &&
                Input.chirpCfg1_idleTimeVar         !== '' &&
                Input.chirpCfg1_adcStartTimeVar     !== '' &&
                Input.chirpCfg1_txEnableMask        !== '') 
            {
                cliOutput = 
                    cliOutput + 'chirpCfg ' + 
                    Input.chirpCfg1_chirpStartIdx   + ' ' +
                    Input.chirpCfg1_chirpEndIdx     + ' ' +
                    Input.chirpCfg1_profileId       + ' ' +
                    Input.chirpCfg1_startFreqVar    + ' ' +
                    Input.chirpCfg1_freqSlopeVar    + ' ' +
                    Input.chirpCfg1_idleTimeVar     + ' ' +
                    Input.chirpCfg1_adcStartTimeVar + ' ' +
                    Input.chirpCfg1_txEnableMask    + '\n';
            } else {
                cliOutput = cliOutput + 'Make sure to provide values for all fields in chirpCfg.\n';
            }
            // chirpCfg 2
            if (widget.ti_widget_container_chirpCfg_2.style.visibility !== 'hidden' &&
                Input.chirpCfg2_chirpStartIdx       !== '' &&
                Input.chirpCfg2_chirpEndIdx         !== '' &&
                Input.chirpCfg2_profileId           !== '' &&
                Input.chirpCfg2_startFreqVar        !== '' &&
                Input.chirpCfg2_freqSlopeVar        !== '' &&
                Input.chirpCfg2_idleTimeVar         !== '' &&
                Input.chirpCfg2_adcStartTimeVar     !== '' &&
                Input.chirpCfg2_txEnableMask        !== '') 
            {
                cliOutput = 
                    cliOutput + 'chirpCfg ' + 
                    Input.chirpCfg2_chirpStartIdx   + ' ' +
                    Input.chirpCfg2_chirpEndIdx     + ' ' +
                    Input.chirpCfg2_profileId       + ' ' +
                    Input.chirpCfg2_startFreqVar    + ' ' +
                    Input.chirpCfg2_freqSlopeVar    + ' ' +
                    Input.chirpCfg2_idleTimeVar     + ' ' +
                    Input.chirpCfg2_adcStartTimeVar + ' ' +
                    Input.chirpCfg2_txEnableMask    + '\n';
            }
            // chirpCfg 3
            if (widget.ti_widget_container_chirpCfg_3.style.visibility !== 'hidden' &&
                Input.chirpCfg3_chirpStartIdx       !== '' &&
                Input.chirpCfg3_chirpEndIdx         !== '' &&
                Input.chirpCfg3_profileId           !== '' &&
                Input.chirpCfg3_startFreqVar        !== '' &&
                Input.chirpCfg3_freqSlopeVar        !== '' &&
                Input.chirpCfg3_idleTimeVar         !== '' &&
                Input.chirpCfg3_adcStartTimeVar     !== '' &&
                Input.chirpCfg3_txEnableMask        !== '') 
            {
                cliOutput = 
                    cliOutput + 'chirpCfg ' + 
                    Input.chirpCfg3_chirpStartIdx   + ' ' +
                    Input.chirpCfg3_chirpEndIdx     + ' ' +
                    Input.chirpCfg3_profileId       + ' ' +
                    Input.chirpCfg3_startFreqVar    + ' ' +
                    Input.chirpCfg3_freqSlopeVar    + ' ' +
                    Input.chirpCfg3_idleTimeVar     + ' ' +
                    Input.chirpCfg3_adcStartTimeVar + ' ' +
                    Input.chirpCfg3_txEnableMask    + '\n';
            }
            // frameCfg
            if (Input.frameCfg_chirpStartIdx        !== '' &&
                Input.frameCfg_chirpEndIdx          !== '' &&
                Input.frameCfg_numLoops             !== '' &&
                Input.frameCfg_numFrames            !== '' &&
                Input.frameCfg_framePeriodicity     !== '' &&
                Input.frameCfg_triggerSelect        !== '' &&
                Input.frameCfg_frameTriggerDelay    !== '') 
            {
                cliOutput = 
                    cliOutput + 'frameCfg ' + 
                    Input.frameCfg_chirpStartIdx    + ' ' +
                    Input.frameCfg_chirpEndIdx      + ' ' +
                    Input.frameCfg_numLoops         + ' ' +
                    Input.frameCfg_numFrames        + ' ' +
                    Input.frameCfg_framePeriodicity + ' ' +
                    Input.frameCfg_triggerSelect    + ' ' +
                    Input.frameCfg_frameTriggerDelay+ '\n';
            } else {
                cliOutput = cliOutput + 'Make sure to provide values for all fields in frameCfg.\n';
            }
            // Append warnings
            if (warnString !== '') cliOutput += '\n\n========== WARNINGS ==========\n' + warnString;
            // Send the cliOutput
            widget.ti_widget_textbox_cliOutput.value = cliOutput;
        }
        else if(device == 'xWRL6432' || device == "xWRL1432"){
            // chirpComnCfg
            if (Input.chirpComnCfg_DigOutputSampRate        !== '' &&
            Input.chirpComnCfg_DigOutputBitsSel             !== '' &&
            Input.chirpComnCfg_DfeFirSel                    !== '' &&
            Input.chirpComnCfg_NumOfAdcSamples              !== '' &&
            Input.chirpComnCfg_ChirpTxMimoPatSel            !== '' &&
            Input.chirpComnCfg_ChirpRampEndTime             !== '' &&
            Input.chirpComnCfg_ChirpRxHpfSel                !== '') 
        {
            cliOutput =
                cliOutput + 'chirpComnCfg ' + 
                Input.chirpComnCfg_DigOutputSampRate        + ' ' +
                Input.chirpComnCfg_DigOutputBitsSel         + ' ' +
                Input.chirpComnCfg_DfeFirSel                + ' ' + 
                Input.chirpComnCfg_NumOfAdcSamples          + ' ' +
                Input.chirpComnCfg_ChirpTxMimoPatSel        + ' ' +
                Input.chirpComnCfg_ChirpRampEndTime         + ' ' +
                Input.chirpComnCfg_ChirpRxHpfSel            + '\n';
        } else {
            cliOutput = cliOutput + 'Make sure to provide values for all fields in chirpComnCfg.\n';
        }
        if (Input.chripTimingCfg_ChirpIdleTime          !== '' &&
            Input.chirpTimingCfg_ChirpAdcSkipSamples    !== '' &&
            Input.chripTimingCfg_ChirpTxStartTime       !== '' &&
            Input.chripTimingCfg_ChirpRfFreqSlope       !== '' &&
            Input.chripTimingCfg_ChirpRfFreqStart       !== '') 
        {
            cliOutput = 
                cliOutput + 'chirpTimingCfg ' + 
                Input.chripTimingCfg_ChirpIdleTime          + ' ' +
                Input.chirpTimingCfg_ChirpAdcSkipSamples    + ' ' +
                Input.chripTimingCfg_ChirpTxStartTime       + ' ' +
                Input.chripTimingCfg_ChirpRfFreqSlope       + ' ' +
                Input.chripTimingCfg_ChirpRfFreqStart       + '\n';
        } else {
            cliOutput = cliOutput + 'Make sure to provide values for all fields in adcbufCfg.\n';
        }
        // channelCfg
        if (Input.channelCfg5_rxChannelEn   !== '' &&
            Input.channelCfg5_txChannelEn   !== '' &&
            Input.channelCfg5_MiscCtrl      !== '') 
        {
            cliOutput = 
                cliOutput + 'channelCfg ' + 
                Input.channelCfg5_rxChannelEn       + ' ' +
                Input.channelCfg5_txChannelEn       + ' ' +
                Input.channelCfg5_MiscCtrl          + '\n';
        } else {
            cliOutput = cliOutput + 'Make sure to provide values for all fields in channelCfg.\n';
        }
        
        //frameCfg
        if (Input.frameCfg5_NumOfChirpsInBurst        !== '' &&
            Input.frameCfg5_NumOfChirpsAccum          !== '' &&
            Input.frameCfg5_BurstPeriodicity          !== '' &&
            Input.frameCfg5_NumOfBurstsInFrame        !== '' &&
            Input.frameCfg5_FramePeriodicity          !== '' &&
            Input.frameCfg5_NumOfFrames               !== '') 
        {
            cliOutput = 
                cliOutput + 'frameCfg ' + 
                Input.frameCfg5_NumOfChirpsInBurst      + ' ' +
                Input.frameCfg5_NumOfChirpsAccum        + ' ' +
                Input.frameCfg5_BurstPeriodicity        + ' ' +
                Input.frameCfg5_NumOfBurstsInFrame      + ' ' +
                Input.frameCfg5_FramePeriodicity        + ' ' +
                Input.frameCfg5_NumOfFrames             + '\n';
        } else {
            cliOutput = cliOutput + 'Make sure to provide values for all fields in frameCfg.\n';
        }
        
        //factoryCalibCfg
        if (Input.factoryCalibCfg_SaveEnable        !== '' &&
            Input.factoryCalibCfg_RestoreEnable          !== '' &&
            Input.factoryCalibCfg_RxGain          !== '' &&
            Input.factoryCalibCfg_TxBackoff        !== '' &&
            Input.factoryCalibCfg_FlashOffset          !== '') 
        {
            cliOutput = 
                cliOutput + 'factoryCalibCfg ' + 
                Input.factoryCalibCfg_SaveEnable      + ' ' +
                Input.factoryCalibCfg_RestoreEnable        + ' ' +
                Input.factoryCalibCfg_RxGain        + ' ' +
                Input.factoryCalibCfg_TxBackoff      + ' ' +
                Input.factoryCalibCfg_FlashOffset        + '\n';
        } else {
            cliOutput = cliOutput + 'Make sure to provide values for all fields in factoryCalibCfg.\n';
        }
        
        
        
        // Append warnings
        if (warnString !== '') cliOutput += '\n\n% ========== WARNINGS ==========\n' + warnString;
        // Send the cliOutput
        widget.ti_widget_textbox_cliOutput.value = cliOutput;
        }
    } else {
        widget.ti_widget_textbox_cliOutput.value = '% ========== ERRORS ==========\n' + errString;
        // Append warnings
        if (warnString !== '') widget.ti_widget_textbox_cliOutput.value += '\n\n% ========== WARNINGS ==========\n' + warnString;
    }
};