// jshint maxerr:1000
/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ===============================================================================================================
   ===============================================================================================================
                            POWER TAB TOOL 
   ===============================================================================================================
   =============================================================================================================== */

/* ====================== USER INPUT ==========================================================================
 * Take user input and the tool's state and pass it to mmWavePower (def in powerToolBackend.js) for processing
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 * General-Case Handling: catch and hand-off input into any other input element in the tool (mostly textboxes)
 *      - Power_onInput(mode)      
 *          - Description:  take input from input cells and pass it to the backend for processing. 
 *                          mode = 'init' used for first time init on loading tool
 *          - Trigger:      input into any non-special-case input
 * ===========================================================================================================
 */
var Power_onInput = function(mode){
    // For ease of reading, get the widget context
    var widget = templateObj.$;

    // Reflect mmWaveAdvanced object's input value in the textboxes & dropdowns:
    if (mode === 'init'){ // Statements to be taken if initializing power tab
        Power_Globals.Inputs.forEach(widgetMap => {
            // Reflect the inputs:
            if (widgetMap[2] == 'dropdown') {
                Util.Html.updateDroplist(widget[widgetMap[0]],mmWavePower.Input[widgetMap[1]]);
                if (widgetMap[1] === 'device') {
                    // Update Dropdown Lists:
                    var evmList     = mmWavePower.get_allowed_evms(mmWavePower.Input.device);
                    if (evmList === false) {
                        console.log('ERROR: Device not in the mmWavePower.Platform object');
                    }
                    else {
                        Util.Html.updateDroplistOptions(widget.ti_widget_droplist_power_evm, evmList);
                        if (evmList.indexOf(widget.ti_widget_droplist_power_evm.selectedValue) < 0){
                            Util.Html.updateDroplist(widget.ti_widget_droplist_power_evm, evmList[evmList.length-1]);
                        }
                    }
                    var siliconList = mmWavePower.get_allowed_silicon_rev(mmWavePower.Input.device, mmWavePower.Input.evm);
                    if (siliconList === false) {
                        console.log('ERROR: Silicon revision not in the mmWavePower.Platform object');
                    }
                    else {
                        Util.Html.updateDroplistOptions(widget.ti_widget_droplist_power_silicon_rev, siliconList);
                        if (siliconList.indexOf(widget.ti_widget_droplist_power_silicon_rev.selectedValue) < 0){
                            Util.Html.updateDroplist(widget.ti_widget_droplist_power_silicon_rev, siliconList[siliconList.length-1]);
                        }
                    }
                }
            }
            else {
                Util.Html.updateTextbox(widget[widgetMap[0]],mmWavePower.Input[widgetMap[1]]);
            }
        });

        if(evm !== 'Custom') {
            widget.ti_widget_droplist_power_mode.disabled = true;
            widget.ti_widget_droplist_power_io_topology.disabled = true;
        }
        
        // Pass the changes to the backend
        mmWavePower.updateInput();
    }
    else { // Statements to be taken when not initializing power tab
        // Object to store all changes to the input fields, passed to mmWavePower.updateInput(...)
        var changes = {}; 
        var evm = templateObj.$.ti_widget_droplist_power_evm.selectedValue;
        
        // reset power mode and io topology to power optimized and 3.3 V when not custom
        if(evm == 'Custom') {
            widget.ti_widget_droplist_power_mode.disabled = false;
            widget.ti_widget_droplist_power_io_topology.disabled = false;
        } else {
            widget.ti_widget_droplist_power_mode.disabled = true;
            Util.Html.updateDroplist(widget.ti_widget_droplist_power_mode, "power");
            changes['mode'] = "power";
            
            widget.ti_widget_droplist_power_io_topology.disabled = true;
            Util.Html.updateDroplist(widget.ti_widget_droplist_power_io_topology, "3.3V");
            changes['io'] = "3.3V";
        }
        
        // Loop through every input field and grab its current value from the front-end
        Power_Globals.Inputs.forEach(widgetMap => {
    
            // int type input fields
            if (widgetMap[2] == 'int') {
                var new_val = parseInt(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    changes[widgetMap[1]] = new_val;
                }
            }
    
            // float type input fields
            else if (widgetMap[2] == 'float') {
                var new_val = parseFloat(widget[widgetMap[0]].value);
                if (isNaN(new_val) === false){
                    changes[widgetMap[1]] = new_val;
                }
            }
    
            // dropdown (string) type input fields
            else {
                var new_val = widget[widgetMap[0]].selectedValue; // String of selected dropdown value
                changes[widgetMap[1]] = new_val;
                
                if (widgetMap[1] === 'device') {
                    // Update Dropdown Lists:
                    var evmList     = mmWavePower.get_allowed_evms(new_val);
                    
                    if (evmList === false) {
                        console.log('ERROR: Device not in the mmWavePower.Platform object');
                    }
                    else {
                        Util.Html.updateDroplistOptions(widget.ti_widget_droplist_power_evm, evmList);
                        if (evmList.indexOf(widget.ti_widget_droplist_power_evm.selectedValue) < 0){
                            Util.Html.updateDroplist(widget.ti_widget_droplist_power_evm, evmList[evmList.length-1]);
                            changes['evm'] = evmList[evmList.length-1];
                        }
                    }
                    var siliconList = mmWavePower.get_allowed_silicon_rev(new_val, widget.ti_widget_droplist_power_evm.selectedValue);
                    if (siliconList === false) {
                        console.log('ERROR: Silicon revision not in the mmWavePower.Platform object');
                    }
                    else {
                        Util.Html.updateDroplistOptions(widget.ti_widget_droplist_power_silicon_rev, siliconList);
                        if (siliconList.indexOf(widget.ti_widget_droplist_power_silicon_rev.selectedValue) < 0){
                            Util.Html.updateDroplist(widget.ti_widget_droplist_power_silicon_rev, siliconList[siliconList.length-1]);
                        }
                    }
                }
                
            }
        });

        // Pass the changes to the backend
        mmWavePower.updateInput(changes);
    }

    // Update the output fields to reflect the changes made in the input fields
    Power_updateOutput();
}
/* ====================== OUTPUT ==========================================================================
 * Take the processed output data from mmWavePower (def in powerToolBackend.js) and update the output fields
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *      - Power_updateOutput()      
 *          - Description:  populate the output fields with the processed results in mmWavePower.Output
 *          - Trigger:      call after Power_onInput() and mmWavePower.updateInput() 
 * 
 *      - Power_updateErrors()
 *          - Description:  based on the tool's state, determined and apply error-checks, create feedback list
 *          - Trigger:      called by Power_updateOutput()
 * 
 *      - Power_generateCliOutput(feedbackStr)
 *          - Description:  take the feedback list from Adv_updateErrors() and create terminal output
 *          - Trigger:      called by Adv_updateOutput() after Adv_updateErrors()
 * ===========================================================================================================
 */
var Power_updateOutput = function() {
    // For ease of reading, get the widget context
    var widget = templateObj.$;
    // Reflect the updated output values computed in the mmWaveAdvanced backend to the GUI frontend
    Power_Globals.Outputs.forEach(widgetMap => {
        if (widgetMap[2] === 'float'){
            Util.Html.updateTextbox(widget[widgetMap[0]], mmWavePower.Output[widgetMap[1]], 3);  // Limit to 3 decimal places
        }
        else{
            Util.Html.updateTextbox(widget[widgetMap[0]], mmWavePower.Output[widgetMap[1]]);
        }
    });

    // Update the error checking
    var feedback = Power_updateErrors();
    Power_generateCliOutput(feedback); // TODO: Enable if/when a terminal is added to this tool
};

var Power_updateErrors = function() {
    var widget = templateObj.$;

    // Strings that hold all the feedback from the error/warning checks
    var errString = '';
    var warnString = '';

    // Remove all error-related cell highlighting then apply it fresh:
    Power_ErrorMaps.Err.forEach(cellName        => Util.Html.highlightPropWithPrecedence(widget[cellName],0,'error1'));
    Power_ErrorMaps.Warn.forEach(cellName       => Util.Html.highlightPropWithPrecedence(widget[cellName],0,'warning'));
    Power_ErrorMaps.Related.forEach(cellName    => Util.Html.highlightPropWithPrecedence(widget[cellName],0,'conditional'));


    // Evaluate the error rules
    Power_ErrorMaps.Err.forEach(map => {
        var widgetNames, isErr, errText;
        widgetNames  = map[0];                  // grab the widgets involved in this error check
        [isErr, errText] = map[1]();            // grab the return value from the error check function call
        if (errText !== '') errString += errText + '\n\n';
        widgetNames.forEach(widgetName => Util.Html.highlightPropWithPrecedence(widget[widgetName],isErr,'error1'));
    });

    // Evaluate the warning rules
    Power_ErrorMaps.Warn.forEach(map => {
        var widgetNames, isErr, warnText;
        widgetNames  = map[0];              // grab the widgets involved in this warning check
        [isErr, warnText] = map[1]();       // grab the return value from the warning check function call
        if (warnText !== '') warnString += '% ' + warnText + '\n\n'; 
        widgetNames.forEach(widgetName => Util.Html.highlightPropWithPrecedence(widget[widgetName],isErr,'warning'));
    });

    return [errString,warnString];
};
var Power_generateCliOutput = function(feedbackStr) {
    var widget = templateObj.$;
    var errString = feedbackStr[0];
    var warnString = feedbackStr[1];

    if (errString !== '')widget.ti_widget_textbox_power_feedback.value = '% ========== ERRORS ==========\n' + errString;
    else if (warnString !== '') widget.ti_widget_textbox_power_feedback.value += '\n\n% ========== WARNINGS ==========\n' + warnString;
    else widget.ti_widget_textbox_power_feedback.value = '*NOTE: Average Power numbers are estimated and are typically within 12% of measured power numbers at nominal temperature (25deg C)'
};